

package fingerworks.fwhid;

import java.io.File;
import java.io.IOException;
import java.util.*;


public final class WinHand2Java {
    static LinkedList handmotion_listeners = new LinkedList();
    static LinkedList gamepad_listeners = new LinkedList();

    /** Must add desired listeners before loading (and thereby permanently entering) the DLL!*/
    public static void enterUSBlistening() {
        System.out.println("About to enter DLL thread (and stay...!)");
        System.loadLibrary("WinHand2Java");
    }
    
    public static void addHandMotionListener(HandMotionListener hml) {
        if(hml != null) {
            synchronized(handmotion_listeners) {
                handmotion_listeners.add(hml);
            }
        }
    }
    public static void addGamePadListener(GamePadListener gpl) {
        if(gpl != null) {
            synchronized(gamepad_listeners) {
                gamepad_listeners.add(gpl);
            }
        }
    }
    public static void removeHandMotionListener(HandMotionListener hml) {
        if(hml != null) {
            synchronized(handmotion_listeners) {
                handmotion_listeners.remove(hml);
            }
        }
    }
    public static void removeGamePadListener(GamePadListener gpl) {
        if(gpl != null) {
            synchronized(gamepad_listeners) {
                gamepad_listeners.remove(gpl);
            }
        }
    }

    public static void incomingHandMotionEvent(int event_type, 
                                          int selectedChord, int lastChord,
                                          int currentSet, int lastSet,
                                          int handActionCode, int slideAxesCode,
                                          int handPressure, int lastPressure,
                                            int handPositionX, int handPositionY,
                                            int Xtrans,
                                            int Ytrans,
                                            int Ztrans,//'zoom'
                                            int Zrot) {//'rotate'
        //System.out.println("incomingHandMotionEvent x" + Xtrans + " y" + Ytrans);
        synchronized(handmotion_listeners) {
            Iterator listiter = handmotion_listeners.iterator();
             while(listiter.hasNext()) {
                 ((HandMotionListener)(listiter.next())).newHandMotion(event_type, 
                                              selectedChord, lastChord,
                                              currentSet, lastSet,
                                              handActionCode,slideAxesCode,
                                              handPressure, lastPressure,
                                              handPositionX, handPositionY,
                                              Xtrans,Ytrans,Ztrans,Zrot);

             }
        }
    }

    public static void incomingGamepadEvent(int base_buttons,
                                            int extra_buttons,
                                            int directional_pad,
                                            int joy1x,
                                            int joy1y,
                                            int joy1twist,
                                            int joy2x,
                                            int joy2y,
                                            int joy2twist
                                        ) {
        //System.out.println("incomingGamepadEvent exbut" + extra_buttons + " x" + joy1x + " y" + joy1y + "rx" + joy2x + " ry" + joy2y);
        synchronized(gamepad_listeners) {
            Iterator listiter = gamepad_listeners.iterator();
             while(listiter.hasNext()) {
                 ((GamePadListener)(listiter.next())).newGamepadState(base_buttons,
                                                extra_buttons,
                                                directional_pad,
                                                joy1x,
                                                joy1y,
                                                joy1twist,
                                                joy2x,
                                                joy2y,
                                                joy2twist
                                            );
             }
        }
    }

    public static void main(String args[]) {
        enterUSBlistening();
    }
}
