/*
*  fwhid_WinHand2Java.c
 *  FingerWorks HID Hand Motion SDK (Windows XP Java-Bridge Implementation)
 *
 *  Created by Wayne Westerman on Dec 9 2003.
 *  Last Updated by Wayne Westerman on Friday Feb 20 2004.
 *  Copyright (c) 2003-2004 FingerWorks, Inc. All rights reserved.
 **/
/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is FWHID_WinHandInternals.c,
*   Windows XP Implementation of Java Bridge for HID Hand Motion SDK.
*
* The Initial Developer of the Original Code is
* Wayne Westerman, FingerWorks, Inc.
*
* Portions created by the Initial Developer are Copyright (C) 2003-2004
* by FingerWorks Inc. All Rights Reserved.
*
* Contributor(s):
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <jni.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "FWHID_HandMotion.h"
#include "fwhid_HandMotionInternals.h"
#include "winxp_HandGameInternals.h"

extern int APIENTRY WinMain (HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmd, int nShow);


JavaVM *cached_jvm = NULL;

static HWND gHwndMain = NULL; //global handle for the main msg window
static HINSTANCE gHInstance = NULL;

char hidheaderstr[256];
char hiddatastr[256];
char hiddevicestr[256];
//char hidrawstr[256];
void fwhdemo_PaintHIDinfo(HWND hWnd, HDC hdc, LPPAINTSTRUCT lpPS) {
	RECT rc;

	GetClientRect(hWnd, &rc);
	DrawText(hdc, hiddevicestr, (int)strlen(hiddevicestr), &rc, DT_CENTER);
	OffsetRect(&rc,0,50); // move the draw position down a bit
	DrawText(hdc, hidheaderstr, (int)strlen(hidheaderstr), &rc, DT_CENTER);
	OffsetRect(&rc,0,100); // move the draw position down a bit
	DrawText(hdc, hiddatastr, (int)strlen(hiddatastr), &rc, DT_CENTER);
	OffsetRect(&rc,0,300);
	//OffsetRect(&rc,0,400);
	//DrawText(hdc, hidrawstr, strlen(hidrawstr), &rc, DT_CENTER);

}


//extern "C" JNIEXPORT void JNICALL
void fwhjava_forwardHandMotion(int handEventID, FWHID_HandMotionPtr pHandMotion) {
	 static jclass cls_s = NULL; /* cached class ID for s */
	 static jmethodID mid_s = NULL;
     JNIEnv *env;
//	printf("forwarding Gamepad Event!\n");
	 fflush(stdout);

	 if(cached_jvm == NULL)
		 return;

  //   printf("got jvm...\n");
     (*cached_jvm)->AttachCurrentThread(cached_jvm, (void **)&env, NULL);
	
	 if(cls_s == NULL) {
		cls_s = (*env)->FindClass(env, "fingerworks/fwhid/WinHand2Java");
		if(cls_s == NULL) {
			return; //exception already thrown
		}
	 }
    // printf("got class...\n");
	 if(mid_s == NULL) {
		mid_s = (*env)->GetStaticMethodID(env, cls_s, "incomingHandMotionEvent", "(IIIIIIIIIIIIIII)V");
		if(mid_s == NULL) {
//			printf("JNI couldn't find incomingHandMotionEvent!\n");
			return; //exception already thrown
		}
	 }
//     printf("callback into Java...\n");
     (*env)->CallStaticVoidMethod(env, cls_s, mid_s,handEventID,
		 pHandMotion->selectedChord,pHandMotion->lastChord,
		 pHandMotion->currentFingers,pHandMotion->lastFingers,
		 pHandMotion->handActionCode,pHandMotion->slideAxesCode,
		 pHandMotion->handPressure,pHandMotion->lastPressure,
		 (int)(400*pHandMotion->handPositionX),(int)(400*pHandMotion->handPositionY),
		 pHandMotion->Xtrans,pHandMotion->Ytrans,
		 pHandMotion->Ztrans,pHandMotion->Zrot);
}

void fwhdemo_displayHandMotion(int handEventID, FWHID_HandMotionPtr pHandMotion) {
	char *bimanualname;
	char *handname;
	char *handeventname;
    char *handevent_typenames[] = {
        "NullHandEvent",
        "ChordSelect",
        "ChordRelease",
        "...reserved...",
        "...reserved...",
        "...reserved...",
        "...reserved...",
        "...reserved...",
        "Pressure",
        "Translate",
        "Rotate",
        "Translate+Rotate",
        "Scale",
        "Translate+Scale",
        "Rotate+Scale",
        "Motion"
    };
	fwhjava_forwardHandMotion(handEventID,pHandMotion);

    if(pHandMotion->selectedChord.flags.bimanualSync) {
        bimanualname = "Bimanual ";
    } else bimanualname = "";

    if(pHandMotion->selectedChord.flags.lefthand) {
        handname = "LeftHand  ";
    } else if(pHandMotion->selectedChord.flags.righthand) {
        handname = "RightHand ";
    } else handname = "NullHand  ";

    if(handEventID >= 0 && handEventID < kEventHandIDmax) {
        handeventname = handevent_typenames[handEventID];
    } else handeventname = "NullEvent ";

    /*    printf(": Schord=0x%x Czone=0x%x pres=%u X=%d Y=%d Z=%d Rz=%d Ry=%d Rz=%d Sx=%d Sy=%d Sz=%d\n",
        pHandMotion->selectedChord.word, pHandMotion->currentZone.word,pHandMotion->handPressure,
        pHandMotion->Xtrans,pHandMotion->Ytrans,pHandMotion->Ztrans,
        pHandMotion->Xrot,pHandMotion->Xrot,pHandMotion->Zrot,
        pHandMotion->Xscale,pHandMotion->Yscale,pHandMotion->Zscale);*/
    wsprintf(hidheaderstr,"%s%s %s: \nSchord=0x%04x Czone=0x%04x\n Pressure=%04u \nX=%d Y=%d Z=%d Rz=%d\n",
			bimanualname, handname,handeventname,
           pHandMotion->selectedChord.word, pHandMotion->currentFingers.word,pHandMotion->handPressure,
           pHandMotion->Xtrans,pHandMotion->Ytrans,pHandMotion->Ztrans,pHandMotion->Zrot);
}


//extern "C" JNIEXPORT void JNICALL
void fwhjava_forwardGamepadEvent(FWHID_GamepadEventPtr pgpe) {
	 static jclass cls_s = NULL; /* cached class ID for s */
	 static jmethodID mid_s = NULL;
     JNIEnv *env;
	 //	printf("forwarding Gamepad Event!\n");
	 //fflush(stdout);

	 if(cached_jvm == NULL)
		 return;

  //   printf("got jvm...\n");
     (*cached_jvm)->AttachCurrentThread(cached_jvm, (void **)&env, NULL);
	
	 if(cls_s == NULL) {
		cls_s = (*env)->FindClass(env, "fingerworks/fwhid/WinHand2Java");
		if(cls_s == NULL) {
			return; //exception already thrown
		}
	 }
    // printf("got class...\n");
	 if(mid_s == NULL) {
		mid_s = (*env)->GetStaticMethodID(env, cls_s, "incomingGamepadEvent", "(IIIIIIIII)V");
		if(mid_s == NULL) {
			return; //exception already thrown
		}
	 }
     //printf("callback into Java...\n");
     (*env)->CallStaticVoidMethod(env, cls_s, mid_s,
		 pgpe->base_buttons,pgpe->extra_buttons,pgpe->directional_pad,
		 pgpe->joy1x,pgpe->joy1y,pgpe->joy1twist,pgpe->joy2x,pgpe->joy2y,pgpe->joy2twist);
}

void fwhdemo_displayGamepadMotion(FWHID_GamepadEventPtr pgpe) {
	int hscroll = 0, vscroll = 0;

	fwhjava_forwardGamepadEvent(pgpe);
    wsprintf(hidheaderstr,"GamepadEvent: \nButtons=0x%04x ExBut=0x%04x Dpad=%d\nX1=%d Y1=%d\nX2=%d Y2=%d\n",
		pgpe->base_buttons, pgpe->extra_buttons, pgpe->directional_pad,
		pgpe->joy1x, pgpe->joy1y, pgpe->joy2x, pgpe->joy2y);
	if(pgpe->joy1x < 124) {
		hscroll = ((int)pgpe->joy1x-124)/20;
	} else if(pgpe->joy1x > 130) {
		hscroll = ((int)pgpe->joy1x-130)/20;
	}
	if(hscroll != 0) {
		HWND win2scroll;
		GUITHREADINFO gti;
		gti.cbSize = sizeof(gti);
		GetGUIThreadInfo(0,&gti); //0 assumes foreground thread
		win2scroll = gti.hwndFocus;
		if(win2scroll != NULL) {
			while(hscroll < 0) {
				SendMessage(win2scroll, WM_HSCROLL, MAKELONG(SB_LINELEFT, 0), 0L);
				hscroll++;
			}
			while(hscroll > 0) {
				SendMessage(win2scroll, WM_HSCROLL, MAKELONG(SB_LINERIGHT, 0), 0L);
				hscroll--;
			}
		}
	}
}



LRESULT CALLBACK fwhdemo_MsgOnlyWndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lParam) {
    PAINTSTRUCT   ps;              /* Also used during window drawing */
	HDC hdc;

    switch (nMsg) {
        case WM_DESTROY:
			FWHID_releaseMultiTouchDevices();
            PostQuitMessage (0);
            return 0;

        case WM_PAINT:	
			hdc = BeginPaint(hwnd, &ps);
			fwhdemo_PaintHIDinfo(hwnd, hdc, &ps);
			EndPaint(hwnd, &ps);
			return 0;

/*		case WM_INPUT:
			//fwhdemo_displayRawHID(lParam);
			FWHID_processHandHRAWINPUT((void *)lParam);

			InvalidateRect(hwnd,0,TRUE);
			SendMessage(hwnd,WM_PAINT,0,0);				
			break;*/
	}		
	//IMPORTANT: let Hand Motion library check/process WM_INPUT or WM_DEVICECHANGED messages
	if(FWHID_WndProc (hwnd, nMsg, wParam, lParam))
		return TRUE;

	return DefWindowProc (hwnd, nMsg, wParam, lParam);
}

HWND fwhdemo_createMsgWindow(HINSTANCE hInst, int nShow) {
	HWND         hwndMain;        /* Handle for the main window. */
    WNDCLASSEX   wndclass;        /* A window class structure. */
    char*        fwMainWndClassName = "FWHID Hand Motion Receiver";/* The name of the main window class */	

    memset (&wndclass, 0, sizeof(WNDCLASSEX));
    wndclass.lpszClassName = fwMainWndClassName;
    wndclass.cbSize = sizeof(WNDCLASSEX);
    wndclass.style = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc = fwhdemo_MsgOnlyWndProc;
    wndclass.hInstance = hInst;
    wndclass.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wndclass.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wndclass.hCursor = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    RegisterClassEx (&wndclass);
    hwndMain = CreateWindow (
            fwMainWndClassName,             /* Class name */
            "Got any Hand Events Lately?",                    /* Caption */
            WS_MINIMIZEBOX | WS_OVERLAPPED | WS_CAPTION |WS_SYSMENU,        /* Style */
            100,//CW_USEDEFAULT,              /* Initial x (use default) */
            100,//CW_USEDEFAULT,              /* Initial y (use default) */
            320,//CW_USEDEFAULT,              /* Initial x size (use default) */
            350,//CW_USEDEFAULT,              /* Initial y size (use default) */
            NULL,                       /* No parent window */
            NULL,                       /* No menu */
            hInst,                      /* This program instance */
            NULL                        /* Creation parameters */
            );
	if(!hwndMain)
		return NULL;

	ShowWindow (hwndMain, nShow);
    UpdateWindow (hwndMain);
	return hwndMain;
}



int APIENTRY
WinMain (HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmd, int nShow) {
		
    MSG          msg;             /* A Win32 message structure. */
	BOOL bRet;

	
	gHwndMain = fwhdemo_createMsgWindow(hInst, nShow);
	
	FWHID_registerMultiTouchDevices(gHwndMain);

	if(nShow == SW_SHOWNORMAL) {
		//set these and don't clear nShow if you want window to display raw hand/gamepad data
		FWHID_setHandEventCallback(fwhdemo_displayHandMotion);
		FWHID_setGamepadEventCallback(fwhdemo_displayGamepadMotion);
	} else {
		FWHID_setHandEventCallback(fwhjava_forwardHandMotion);
		FWHID_setGamepadEventCallback(fwhjava_forwardGamepadEvent);
	}
	

	while( (bRet = GetMessage( &msg, NULL, 0, 0 )) != 0) {
		if (bRet == -1) {
			// handle the error and possibly exit
		} else {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

    return (int)msg.wParam;
}




BOOL APIENTRY DllMain( HINSTANCE hInstance,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
                     ) {

	if ( ul_reason_for_call == DLL_PROCESS_ATTACH ) {
		gHInstance = hInstance;
		//WinMain(hInstance,NULL,NULL,SW_SHOWNORMAL);
    }
    /*
    if(ul_reason_for_call == DLL_PROCESS_DETACH)    	
    	//printf("jusbFWin - Native library detached from process.\n");

    if(ul_reason_for_call == DLL_THREAD_ATTACH)
        //printf("jusbFWin - Native library attached to thread.\n");

    if(ul_reason_for_call == DLL_THREAD_DETACH)    	
        printf("jusbFWin - Native library detached from thread.\n");
    */
    return TRUE;
}


DWORD WINAPI EventThreadFunc(LPVOID lpParam) {
	int nShow = SW_SHOWNORMAL;
	//comment out the line below if you want to display native window for raw hand/gamepad data
	nShow = SW_HIDE;

	return WinMain(gHInstance, NULL, NULL, nShow);
}


JNIEXPORT jint JNICALL JNI_OnLoad(JavaVM *jvm, void *reserved) {
	int nShow = SW_SHOWNORMAL;
	DWORD dwThreadId;
	HANDLE hThread;

	cached_jvm = jvm;
	//printf("JNI_OnLoad");
	//fflush(stdout);

	//comment out the line below if you want to display native window for raw hand/gamepad data
	nShow = SW_HIDE;

	//WinMain(gHInstance,NULL,NULL,nShow);
	hThread = CreateThread(NULL,0,EventThreadFunc,
						gHInstance,0,&dwThreadId);

	return JNI_VERSION_1_2;
}
