/*
 *  winxp_HandMotionInternals.c
 *  FingerWorks HID Hand Motion SDK (Windows XP Implementation)
 *
 *  Created by Wayne Westerman on Dec 9 2003.
 *  Last Updated by Wayne Westerman on Friday Feb 20 2003.
 *  Copyright (c) 2003 FingerWorks, Inc. All rights reserved.
 **/
 /* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is win32_HandMotionInternals.c,
 *   Windows XP Implementation of FingerWorks HID Hand Motion SDK.
 *
 * The Initial Developer of the Original Code is
 * Wayne Westerman, FingerWorks, Inc.
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * by FingerWorks Inc. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// system includes ----------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

// project includes ---------------------------------------------------------
#include "winxp_HandGameInternals.h"
#include "FWHID_HandMotion.h"

//pre-compilation options ---------------------------------------------------

#define FWHAND_DEBUG_NEW_DEVICES 0
// prototypes ---------------------------------------------------------------


// static functions (internal/private) ----------------------------------------------

FWHIDJoystickEventCALLBACK gJoystickEventCallbackp = NULL;

// functions (external/public) ----------------------------------------------

void FWHID_setJoystickEventCallback(FWHIDJoystickEventCALLBACK fwcallback) {
	gJoystickEventCallbackp = fwcallback;
}


static void fwhand_parseChoemJoysticks(BYTE *bytearray, DWORD numbytes) {
	FWHID_GamepadEvent gpe;
	if(bytearray != NULL && numbytes >= 16) {
		gpe.base_buttons = bytearray[13];
		gpe.extra_buttons = bytearray[14];
		gpe.joy1x = bytearray[7] | ((bytearray[8] << 8) & 0xF00);
		gpe.joy1y = bytearray[9] | ((bytearray[10] << 8) & 0xF00);
		gpe.joy1twist = bytearray[11] | ((bytearray[12] << 8) & 0xF00);
		gpe.joy2x = bytearray[1] | ((bytearray[2] << 8) & 0xF00);
		gpe.joy2y =  bytearray[3] | ((bytearray[4] << 8) & 0xF00);
		gpe.joy2twist = bytearray[5] | ((bytearray[6] << 8) & 0xF00);
		gpe.directional_pad = 0;
		if(gJoystickEventCallbackp != NULL) {
			(*gJoystickEventCallbackp)(&gpe);
		}
	}
}


FWHIDGamepadEventCALLBACK gGamepadEventCallbackp = NULL;

// functions (external/public) ----------------------------------------------

void FWHID_setGamepadEventCallback(FWHIDGamepadEventCALLBACK fwcallback) {
	gGamepadEventCallbackp = fwcallback;
}


static void fwhand_parseBelkinNostromoGamepad(BYTE *bytearray, DWORD numbytes) {
	FWHID_GamepadEvent gpe;
	if(bytearray != NULL && numbytes >= 8) {
		gpe.base_buttons = bytearray[6];
		gpe.extra_buttons = bytearray[7];
		gpe.joy1x = bytearray[1];
		gpe.joy1y = bytearray[2];
		gpe.joy2y = bytearray[3];
		gpe.joy2x = bytearray[4];
		gpe.directional_pad = bytearray[5];
		if(gGamepadEventCallbackp != NULL) {
			(*gGamepadEventCallbackp)(&gpe);
		}
	}
}

static void fwhand_parseGravisEliminatorGamepad(BYTE *bytearray, DWORD numbytes) {
	FWHID_GamepadEvent gpe;
	if(bytearray != NULL && numbytes >= 5) {
		gpe.base_buttons = bytearray[3];
		gpe.extra_buttons = bytearray[4];
		gpe.joy1x = bytearray[1];
		gpe.joy1y = bytearray[2];
		gpe.joy2y = 0;
		gpe.joy2x = 0;
		gpe.directional_pad = 0;
		if(gGamepadEventCallbackp != NULL) {
			(*gGamepadEventCallbackp)(&gpe);
		}
	}
}

BOOL fwhand_requestRawInput(USHORT hid_usage_page, USHORT hid_usage_num, DWORD event_handling_options, HWND hwndTarget)
{
	RAWINPUTDEVICE Rid;

	Rid.usUsagePage = hid_usage_page;
	Rid.usUsage = hid_usage_num;
	Rid.dwFlags = event_handling_options;
	Rid.hwndTarget = hwndTarget;

	return RegisterRawInputDevices(&Rid, 1, sizeof (Rid)) != FALSE;
}

static UINT HIDCountDevices() {
	RAWINPUTDEVICE rids[128];
	UINT nrids = 128;
	return GetRegisteredRawInputDevices(rids,&nrids,sizeof(RAWINPUTDEVICE));
}


// functions (external/public) ----------------------------------------------

static bool FWHID_processHandHRAWINPUT(void* hrawinput) {
	static LPBYTE lpb = NULL;
	static UINT size_of_lpb=0;
	UINT dwSupposedSize;
	UINT dwSize;
	RAWINPUT *raw;

	GetRawInputData((HRAWINPUT)hrawinput, RID_INPUT, NULL, &dwSupposedSize,sizeof(RAWINPUTHEADER));

	if(dwSupposedSize > size_of_lpb){
		size_of_lpb = dwSupposedSize;
		lpb = realloc(lpb,sizeof(LPBYTE) * size_of_lpb *2); //Why x2??????
	}

	if (lpb == NULL) {
		return FALSE;
	}
	dwSize = dwSupposedSize;
	if (GetRawInputData((HRAWINPUT)hrawinput, RID_INPUT, lpb, &dwSize,
		sizeof(RAWINPUTHEADER)) != dwSupposedSize )
		OutputDebugString (TEXT("GetRawInputData doesn't return correct size !\n"));

	raw = (RAWINPUT*)lpb;

	if (raw->header.dwType == RIM_TYPEHID) {
		int pcbsize;
		RID_DEVICE_INFO devinfo;
							
		pcbsize = devinfo.cbSize = sizeof(RID_DEVICE_INFO);
			
		GetRawInputDeviceInfo(raw->header.hDevice, RIDI_DEVICEINFO, &devinfo, &pcbsize);
		if(devinfo.hid.usUsagePage == kFWHIDPage_GenericDesktop) {
			if (devinfo.hid.dwVendorId == kFWHID_FINGERWORKS_USB_VENDOR_ID) {
				if(devinfo.hid.usUsage == kFWHIDUsage_GD_MultiAxisController) {
					unsigned packetnum = 0;
					for(packetnum = 0; packetnum < raw->data.hid.dwCount; packetnum++) {
						fwhand_parseRawHandMotion(&raw->data.hid.bRawData[packetnum*raw->data.hid.dwSizeHid], raw->data.hid.dwSizeHid, devinfo.hid.dwVersionNumber);
					}
				} else {
					//OutputDebugString (TEXT("Unknown FingerWorks Report!\n"));
				}
				return TRUE;
			} else if (devinfo.hid.dwVendorId == kFWHID_BELKIN_USB_VENDOR_ID
				&& devinfo.hid.usUsage == kFWHIDUsage_GD_GamePad
				&& devinfo.hid.dwProductId == kFWHID_BELKIN_NOSTRAMO_USB_PRODUCT_ID) {
				//play with joysticks
				unsigned packetnum = 0;
				for(packetnum = 0; packetnum < raw->data.hid.dwCount; packetnum++) {
					fwhand_parseBelkinNostromoGamepad(&raw->data.hid.bRawData[packetnum*raw->data.hid.dwSizeHid], raw->data.hid.dwSizeHid);
				}
				//OutputDebugString (TEXT("Belkin HID report!\n"));
				return TRUE;
			} else if (devinfo.hid.dwVendorId == kFWHID_GRAVIS_USB_VENDOR_ID
				&& devinfo.hid.usUsage == kFWHIDUsage_GD_GamePad
				&& devinfo.hid.dwProductId == kFWHID_GRAVIS_ELIMINATOR_USB_PRODUCT_ID) {
				//play with joysticks
				unsigned packetnum = 0;
				for(packetnum = 0; packetnum < raw->data.hid.dwCount; packetnum++) {
					fwhand_parseGravisEliminatorGamepad(&raw->data.hid.bRawData[packetnum*raw->data.hid.dwSizeHid], raw->data.hid.dwSizeHid);
				}
				//OutputDebugString (TEXT("Gravis HID report!\n"));
				return TRUE;
			} else if (devinfo.hid.dwVendorId == kFWHID_CHOEM_USB_VENDOR_ID
				&& devinfo.hid.usUsage == kFWHIDUsage_GD_Joystick
				&& devinfo.hid.dwProductId == kFWHID_CHOEM_JOYSTICKBOX_USB_PRODUCT_ID) {
				//play with joysticks
				unsigned packetnum = 0;
				for(packetnum = 0; packetnum < raw->data.hid.dwCount; packetnum++) {
					fwhand_parseChoemJoysticks(&raw->data.hid.bRawData[packetnum*raw->data.hid.dwSizeHid], raw->data.hid.dwSizeHid);
				}
				//OutputDebugString (TEXT("Gravis HID report!\n"));
				return TRUE;
			}
		} else {
			//ignore
			//OutputDebugString (TEXT("Unknown HID report!\n"));
		}
	} else if(raw->header.dwType == RIM_TYPEMOUSE) {
		//OutputDebugString (TEXT("Mouse Report!\n"));
	}
	//free(lpb);  //no longer freeing due to use of realloc with static pointer
	return FALSE;
}


LRESULT CALLBACK FWHID_WndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lParam) {
    switch (nMsg) {
		case WM_INPUT:
			return FWHID_processHandHRAWINPUT((void*)lParam);
	}		
	return FALSE;
}


//call this when your application starts to initialize connections to the FingerWorks device
//  targetWindow parameter must be valid HWND for Windows implementation
void FWHID_registerMultiTouchDevices (void* targetWindow) {
    fwhand_clearState();

    if(FWHID_getHandEventCallback() == NULL) //no callback installed yet, so assume just print!
        FWHID_setHandEventCallback(FWHID_printHandMotion);  //intall default FWHIDHandEventCALLBACK

	fwhand_requestRawInput(kFWHIDPage_GenericDesktop, kFWHIDUsage_GD_GamePad, RIDEV_INPUTSINK, (HWND)targetWindow);
	fwhand_requestRawInput(kFWHIDPage_GenericDesktop, kFWHIDUsage_GD_MultiAxisController, RIDEV_INPUTSINK, (HWND)targetWindow);
	fwhand_requestRawInput(kFWHIDPage_GenericDesktop, kFWHIDUsage_GD_Joystick, RIDEV_INPUTSINK, (HWND)targetWindow);
	//fwhand_requestRawInput(kFWHIDPage_GenericDesktop, kFWHIDUsage_GD_Mouse, RIDEV_INPUTSINK, (HWND)targetWindow);
#if FWHAND_DEBUG_NEW_DEVICES
	printf("--- Found %u Matching FWHID Devices ---\n",HIDCountDevices());
	fflush (stdout);                	
#endif //FWHAND_DEBUG_NEW_DEVICES
}

void FWHID_releaseMultiTouchDevices (void) {
#if FWHAND_DEBUG_NEW_DEVICES
    printf("\n--- Releasing %u FWHID Hand Motion Devices ---\n",HIDCountDevices ());
    fflush (stdout);
#endif //FWHAND_DEBUG_NEW_DEVICES
}

