/*
 *  macosx_HandMotionInternals.c
 *  FingerWorks HID Hand Motion SDK (Mac OS X Implementation)
 *
 *  Created by Wayne Westerman on Wed Jun 25 2003.
 *  Last Updated by Wayne Westerman on Friday Feb 20 2004.
 *  Copyright (c) 2003-2004 FingerWorks, Inc. All rights reserved. 
 **/
 /* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FWHID_HandMotion.c, 
 *   Mac OS X Implementation of FingerWorks HID Hand Motion SDK.
 *
 * The Initial Developer of the Original Code is
 * Wayne Westerman, FingerWorks, Inc.
 *
 * Portions created by the Initial Developer are Copyright (C) 2003-2004
 * by FingerWorks Inc. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// system includes ----------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>

// project includes ---------------------------------------------------------
#include "HID_Utilities_External.h"
#include "FWHID_HandMotion.h"
#include "macosx_HandMotionInternals.h"

//pre-compilation options ---------------------------------------------------

#define FWHAND_DEBUG_NEW_DEVICES 0
// prototypes ---------------------------------------------------------------


// static functions (internal/private) ----------------------------------------------

static void fwhand_printCookie(char *usage_name, IOHIDElementCookie new_cookie) {
#if FWHAND_DEBUG_NEW_DEVICES     
    printf("--- Found %s cookie [0x%lx] ----\n",usage_name,(unsigned long)new_cookie);
    //fflush(stdout);
#endif // FWHAND_DEBUG_NEW_DEVICES     
}

static void fwhand_updateMinMaxCookies(FWHID_HandMotionReconstructionPtr pHandState, IOHIDElementCookie new_cookie) {
    if(pHandState != NULL) {
        if(pHandState->min_cookie == 0 ||
           pHandState->min_cookie > new_cookie) {
            pHandState->min_cookie = new_cookie;
        }
        if(pHandState->max_cookie == 0 ||
           pHandState->max_cookie < new_cookie) {
            pHandState->max_cookie = new_cookie;
        }
    }
}


void fwhand_storeHandMotionCookies(pRecDevice pHIDDevice, FWHID_HandMotionReconstructionPtr pLeftHandState, 
														  FWHID_HandMotionReconstructionPtr pRightHandState) {
    pRecElement pElement;
    FWHID_HandMotionReconstructionPtr pMotionReconstructor = NULL;        
    memset(pLeftHandState,0,sizeof(FWHID_HandMotionReconstruction));
    memset(pRightHandState,0,sizeof(FWHID_HandMotionReconstruction));
    
    for(pElement = HIDGetFirstDeviceElement(pHIDDevice,kHIDElementTypeAll);
        pElement != NULL;
        pElement = HIDGetNextDeviceElement(pElement,kHIDElementTypeAll)) {
        if (pElement->usagePage == kHIDPage_GenericDesktop) {
            if(pElement->type == kIOHIDElementTypeCollection) {
#if FWHAND_DEBUG_NEW_DEVICES     
                printf("--- Collection usage page 0x%lx, 0x%lx usage ----\n",pElement->usagePage,pElement->usage);
#endif //FWHAND_DEBUG_NEW_DEVICES                
                if(pElement->usage == kFWHIDUsage_GD_RightHandMotion) { 
#if FWHAND_DEBUG_NEW_DEVICES     
					printf("---    (RightHand Collection)\n");
#endif //FWHAND_DEBUG_NEW_DEVICES                
                    pMotionReconstructor = pRightHandState;
                } else if(pElement->usage == kFWHIDUsage_GD_LeftHandMotion) {
#if FWHAND_DEBUG_NEW_DEVICES     
					printf("---    (LeftHand Collection)\n");
#endif //FWHAND_DEBUG_NEW_DEVICES                
                    pMotionReconstructor = pLeftHandState;
                } else {
#if FWHAND_DEBUG_NEW_DEVICES     
					printf("---    (NullHand Collection)\n");
#endif //FWHAND_DEBUG_NEW_DEVICES                				
					pMotionReconstructor = NULL;
				}
            } else if(pMotionReconstructor == NULL) {
#if FWHAND_DEBUG_NEW_DEVICES                
                printf("--- Unexpected GD element type! usage page 0x%lx, 0x%lx usage----\n",pElement->usagePage,pElement->usage);
#endif //FWHAND_DEBUG_NEW_DEVICES                
                //ignore feature/output elements
                //(long)HIDGetPreviousDeviceElement (pElement, kHIDElementTypeCollection));
            } else if(pElement->usage == kHIDUsage_GD_X) {
                pMotionReconstructor->elements.x = pElement;
                pMotionReconstructor->cookies.x = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("X",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Y) {
                pMotionReconstructor->elements.y = pElement;
                pMotionReconstructor->cookies.y = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("Y",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Z) {
                pMotionReconstructor->elements.z = pElement;
                pMotionReconstructor->cookies.z = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("Z",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Rx) {
                pMotionReconstructor->elements.Rx = pElement;
                pMotionReconstructor->cookies.Rx = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("Rx",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Ry) {
                pMotionReconstructor->elements.Ry = pElement;
                pMotionReconstructor->cookies.Ry = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("Ry",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Rz) {
                pMotionReconstructor->elements.Rz = pElement;
                pMotionReconstructor->cookies.Rz = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("Rz",pElement->cookie);
            } else if(pElement->usage == kFWHIDUsage_GD_Sx) {
                pMotionReconstructor->elements.Sx = pElement;
                pMotionReconstructor->cookies.Sx = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("Sx",pElement->cookie);
            } else if(pElement->usage == kFWHIDUsage_GD_Sy) {
                pMotionReconstructor->elements.Sy = pElement;
                pMotionReconstructor->cookies.Sy = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("Sy",pElement->cookie);
            } else if(pElement->usage == kFWHIDUsage_GD_Sz) {
                pMotionReconstructor->elements.Sz = pElement;
                pMotionReconstructor->cookies.Sz = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("Sz",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Vno) {
                pMotionReconstructor->elements.HandPressure = pElement;
                pMotionReconstructor->cookies.HandPressure = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("HandPressure (Vno)",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Vx) {
                pMotionReconstructor->elements.HandPositionX = pElement;
                pMotionReconstructor->cookies.HandPositionX = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("HandPositionX (Vx)",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Vy) {
                pMotionReconstructor->elements.HandPositionY = pElement;
                pMotionReconstructor->cookies.HandPositionY = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("HandPositionY (Vy)",pElement->cookie);
            } else if(pElement->usage == kFWHIDUsage_GD_StartChord) {
                pMotionReconstructor->elements.StartChord = pElement;
                pMotionReconstructor->cookies.StartChord = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("StartChord",pElement->cookie);
            } else if(pElement->usage == kFWHIDUsage_GD_CurrentFingers) {
                pMotionReconstructor->elements.CurrentFingers = pElement;
                pMotionReconstructor->cookies.CurrentFingers = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("CurrentFingers",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_MotionWakeup) {
                pMotionReconstructor->elements.SlideAxesCode = pElement;
                pMotionReconstructor->cookies.SlideAxesCode = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("SlideAxesCode",pElement->cookie);
            } else if(pElement->usage == kHIDUsage_GD_Hatswitch) {
                pMotionReconstructor->elements.HandActionCode = pElement;
                pMotionReconstructor->cookies.HandActionCode = pElement->cookie;
                fwhand_updateMinMaxCookies(pMotionReconstructor, pElement->cookie);
                fwhand_printCookie("HandActionCode",pElement->cookie);
            } else printf("--- Found unexpected 0x%lx generic desktop usage ----\n",pElement->usage);
        } else if(pElement->usagePage == kHIDPage_Digitizer){
        } else if(pElement->usagePage == kHIDPage_AlphanumericDisplay){            
        } else if(pElement->usagePage == kFWHIDPage_ProximityImages) {    
        } else printf ("Error unexpected usage page 0x%lx for input element",pElement->usagePage);        
    }
}                        



static void fwhand_reconstructMotionElements(FWHID_HandMotionReconstructionPtr pHandState,
                                          IOHIDEventStruct *pHIDEvent) {
    IOHIDElementCookie newCookie = pHIDEvent->elementCookie;
    SInt32 value = pHIDEvent->value;
    if (newCookie == pHandState->cookies.StartChord) {
        fwhand_processChordChange(&pHandState->motion,value & 0xFFFF);
		//printf("    Got StartChord cookie [0x%lx] value=0x%lx\n",(unsigned long)newCookie, (unsigned long)value);    
    } else if(newCookie == pHandState->cookies.CurrentFingers) {   
        pHandState->motion.lastFingers.word = pHandState->motion.currentFingers.word;
        pHandState->motion.currentFingers.fields.fingers = value & 0xFF;
		if(pHandState->elements.HandActionCode == NULL) //in v <= 1.40 handActionCode is in upper byte 
			pHandState->motion.handActionCode = (value & 0xFF00) >> 8;
		else if(pHandState->elements.SlideAxesCode == NULL) //in v > 1.40 slideAxesCode is in upper byte
			pHandState->motion.slideAxesCode = (value & 0xFF00) >> 8;
        //printf("    Got CurrentZone cookie [0x%lx] value=0x%lx\n",(unsigned long)newCookie, (unsigned long)value);    
    } else if(newCookie == pHandState->cookies.HandPressure) {   
        pHandState->motion.lastPressure = pHandState->motion.handPressure;
        pHandState->motion.handPressure = value & 0xFFFF;
    } else if(newCookie == pHandState->cookies.HandPositionX) {   
        pHandState->motion.handPositionX = value/400.0; //convert 25um units to cm
    } else if(newCookie == pHandState->cookies.HandPositionY) {   
        pHandState->motion.handPositionY = value/400.0; //convert 25um units to cm
        //printf("    Got HandPressure cookie [0x%lx] value=0x%lx\n",(unsigned long)newCookie, (unsigned long)value);    
    } else if(newCookie == pHandState->cookies.x) {   
        pHandState->motion.Xtrans += value;
        //printf("    Got X %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.y) {   
        pHandState->motion.Ytrans += value;
        //printf("    Got Y %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.z) {   
        pHandState->motion.Ztrans += value;
        //printf("    Got Z %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.Rx) {   
        pHandState->motion.Xrot += value;
        //printf("    Got Rx %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.Ry) {   
        pHandState->motion.Yrot += value;
        //printf("    Got Ry %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.Rz) {   
        pHandState->motion.Zrot += value;
        //printf("    Got Rz %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.Sx) {   
        pHandState->motion.Xscale += value;
        //printf("    Got Sx %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.Sy) {   
        pHandState->motion.Yscale += value;
        //printf("    Got Sy %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.Sz) {   
        pHandState->motion.Zscale += value;
        //printf("    Got Sz %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    } else if(newCookie == pHandState->cookies.SlideAxesCode
				&& pHandState->elements.SlideAxesCode != NULL) {   
        pHandState->motion.slideAxesCode = value & 0xFF;
    } else if(newCookie == pHandState->cookies.HandActionCode
		&& pHandState->elements.HandActionCode != NULL) {   
        pHandState->motion.handActionCode = value & 0xFF;
        //printf("    Got Sz %ld [cookie 0x%lx]\n", value,(unsigned long)newCookie);    
    }
}               

bool fwhand_handleHIDEvent(IOHIDEventStruct *eventP, FWHID_HandMotionReconstructionPtr pLeftHandState, 
														  FWHID_HandMotionReconstructionPtr pRightHandState) {
	if(pRightHandState != NULL 
	   && eventP->elementCookie >= pRightHandState->min_cookie 
	   && eventP->elementCookie <= pRightHandState->max_cookie) {
		fwhand_reconstructMotionElements(pRightHandState,eventP);                            
		return true;
	}
	if(pLeftHandState != NULL 
	   && eventP->elementCookie >= pLeftHandState->min_cookie 
	   && eventP->elementCookie <= pLeftHandState->max_cookie) {
		fwhand_reconstructMotionElements(pLeftHandState,eventP);                            
		return true;
	}
	return false;
}

void fwhand_finishHIDEventBatch(FWHID_HandMotionReconstructionPtr pLeftHandState, 
														  FWHID_HandMotionReconstructionPtr pRightHandState) {
	if(pLeftHandState != NULL)													  
		fwhand_processAccumulatedMotion(&pLeftHandState->motion);
	if(pRightHandState != NULL)													  
		fwhand_processAccumulatedMotion(&pRightHandState->motion);
}


