/*
 *  HandPathTest.c
 *  FingerWorks HID Contact Path SDK (Windows XP Demonstration)
 *
 *  Created byHai Zhu  Sept 29 2003.
 *  Last Updated by James Orr on Aug 25 2004.
 *  Copyright (c) 2003 FingerWorks, Inc. All rights reserved.
 **/

//#define _WIN32_WINNT 0x0501   // Identify this as a Windows XP application.  This is necessary
                              //    to pull in RawInput (which is an XP exclusive)

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
//#include "FWHID_HandMotion.h"
#include "FWHID_MultiTouchDevice.h"

static HWND gHwndMain = NULL; //global handle for the main msg window

FILE *pathlogfile = NULL;
char hidheaderstr[256];
char hiddatastr[256];
char hiddevicestr[256];

void fwhdemo_PaintHIDinfo(HWND hWnd, HDC hdc, LPPAINTSTRUCT lpPS) {
	RECT rc;

	GetClientRect(hWnd, &rc);
	DrawText(hdc, hiddevicestr, (int)strlen(hiddevicestr), &rc, DT_CENTER);
	OffsetRect(&rc,0,50); // move the draw position down a bit
	DrawText(hdc, hidheaderstr, (int)strlen(hidheaderstr), &rc, DT_CENTER);
	OffsetRect(&rc,0,100); // move the draw position down a bit
	DrawText(hdc, hiddatastr, (int)strlen(hiddatastr), &rc, DT_CENTER);
	OffsetRect(&rc,0,300);
	//OffsetRect(&rc,0,400);
	//DrawText(hdc, hidrawstr, strlen(hidrawstr), &rc, DT_CENTER);

}


void fwhdemo_displayHandMotion(int handEventID, FWHID_HandMotionPtr pHandMotion) {
	char *bimanualname;
	char *handname;
	char *handeventname;
    char *handevent_typenames[] = {
        "NullHandEvent",
        "ChordSelect",
        "ChordRelease",
        "...reserved...",
        "...reserved...",
        "...reserved...",
        "...reserved...",
        "...reserved...",
        "Pressure",
        "Translate",
        "Rotate",
        "Translate+Rotate",
        "Scale",
        "Translate+Scale",
        "Rotate+Scale",
        "Motion"
    };

    if(pHandMotion->selectedChord.flags.bimanualSync) {
        bimanualname = "Bimanual ";
    } else bimanualname = "";

    if(pHandMotion->selectedChord.flags.lefthand) {
        handname = "LeftHand  ";
    } else if(pHandMotion->selectedChord.flags.righthand) {
        handname = "RightHand ";
    } else handname = "NullHand  ";

    if(handEventID >= 0 && handEventID < kEventHandIDmax) {
        handeventname = handevent_typenames[handEventID];
    } else handeventname = "NullEvent ";

    /*    printf(": Schord=0x%x Czone=0x%x pres=%u X=%d Y=%d Z=%d Rz=%d Ry=%d Rz=%d Sx=%d Sy=%d Sz=%d\n",
        pHandMotion->selectedChord.word, pHandMotion->currentZone.word,pHandMotion->handPressure,
        pHandMotion->Xtrans,pHandMotion->Ytrans,pHandMotion->Ztrans,
        pHandMotion->Xrot,pHandMotion->Xrot,pHandMotion->Zrot,
        pHandMotion->Xscale,pHandMotion->Yscale,pHandMotion->Zscale);*/
    wsprintf(hidheaderstr,"%s%s %s: \nSchord=0x%04x Czone=0x%04x\n Pressure=%04u \nX=%d Y=%d Z=%d Rz=%d\n",
			bimanualname, handname,handeventname,
           pHandMotion->selectedChord.word, pHandMotion->currentFingers.word,pHandMotion->handPressure,
           pHandMotion->Xtrans,pHandMotion->Ytrans,pHandMotion->Ztrans,pHandMotion->Zrot);
	OutputDebugString(hidheaderstr);
}

void fwhdemo_displayContact(FWMultiTouchDevicePtr fwdevice,
							void *targetData) {
	char debugstr[256];
	FWHID_ContactPtr pContact = (FWHID_ContactPtr)targetData;

    wsprintf(hiddatastr,"XPos=%d  YPos=%d  Area=%d\nPathID=%d  FingerID=%d  HandID=%d\n",
		(int)(pContact->xpos*400), (int)(pContact->ypos*400), (int)(pContact->proximity*100),
		pContact->path_id, pContact->finger_id, pContact->hand_id);

	InvalidateRect(gHwndMain,0,TRUE);
	SendMessage(gHwndMain,WM_PAINT,0,0);				

	FWHID_snprintfContact(debugstr,256,pContact);
	/*sprintf(debugstr,"Fr#%d P%d Fing%d Hand=%d x=%.2f y=%.2f xvel=%.2f yvel=%.2f prox=%.2f orien=%.1f eccen=%.2f\n",
           pContact->frame_no, pContact->path_id, pContact->finger_id, pContact->hand_id,
           pContact->xpos, pContact->ypos, pContact->xvel, pContact->yvel,pContact->proximity,    	
		   pContact->orientation,pContact->eccentricity);*/
	
	OutputDebugString (TEXT(debugstr));

}

void fwpdemo_printPathFrame(FWMultiTouchDevicePtr fwdevice,
							void *targetData) {
	char debugstr[4096];

	FWHID_snprintfActivePaths(debugstr,4096,fwdevice,false);
	
	OutputDebugString (TEXT(debugstr));
	if(pathlogfile != NULL) {
		fputs(debugstr,pathlogfile);
		//fputs("\n",pathlogfile); //put blank line between frames
		fflush(pathlogfile);
	}
}

void fwhdemo_RemovingMultiTouchDevice(FWMultiTouchDevicePtr fwdevice,
									 void *targetData) {
	OutputDebugString (TEXT("Got MultiTouch DeviceDisposal Callback!\n"));
	if(pathlogfile != NULL) {
		fclose(pathlogfile);
	}

	if(FWHID_disableStreams(fwdevice,kFW_ContactPathStream))
		OutputDebugString (TEXT("Path stream disabled!\n"));
	else OutputDebugString (TEXT("Either this is not a path interface or path stream disabling failed!\n"));
}

void fwhdemo_MultiTouchDeviceCreated(FWMultiTouchDevicePtr fwdevice,
									 void *targetData) {
	//FWHID_setContactCallback(fwdevice,fwhdemo_displayContact);
	//FWHID_setContactFrameCallback(fwdevice,??);
	FWHID_setPathFrameCallback(fwdevice,fwpdemo_printPathFrame);
	FWHID_setHandEventCallback(fwhdemo_displayHandMotion);

	OutputDebugString (TEXT("Got MultiTouch DeviceCreation Callback..opening logfile!\n"));
	pathlogfile = fopen("contactpathlog.txt","w");

	if(FWHID_enableStreams(fwdevice,kFW_ContactPathStream))
		OutputDebugString (TEXT("Path stream enabled!\n"));
	else OutputDebugString (TEXT("Either this is not a path interface or path stream enabling failed!\n"));
}

#define WM_UNREGISTER_HANDLE WM_USER+1

LRESULT CALLBACK fwhdemo_MsgOnlyWndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lParam) {    
    PAINTSTRUCT   ps;              /* Also used during window drawing */
	HDC hdc;

    switch (nMsg) {
        case WM_DESTROY:
			FWHID_releaseMultiTouchDevices();
            PostQuitMessage (0);
            return 0;

        case WM_PAINT:	    
			hdc = BeginPaint(hwnd, &ps);
			fwhdemo_PaintHIDinfo(hwnd, hdc, &ps);
			EndPaint(hwnd, &ps);
			return 0;

		//case WM_ERASEBKGND:
		//	return (LRESULT)1; // Say we handled it.


		/*case WM_INPUT:
			//fwhdemo_displayRawHID(lParam);
			FWHID_processMultiTouchHRAWINPUT((void *)lParam);

			InvalidateRect(hwnd,0,TRUE);
			SendMessage(hwnd,WM_PAINT,0,0);				
			break;
		case WM_DEVICECHANGE:
		case WM_UNREGISTER_HANDLE:
			{
			char buffer[256];
			sprintf(buffer,"WNDPROC: msg: %d wParam: %d lParam: %d\n",nMsg,wParam,lParam);
			OutputDebugString(buffer);
			}
			if(FWHID_WndProc(hwnd, nMsg, wParam,lParam)) return TRUE;
			break;*/
	}		

	//IMPORTANT: let FWHID library check/process WM_INPUT or WM_DEVICECHANGED messages
	if(FWHID_WndProc (hwnd, nMsg, wParam, lParam)) return TRUE;

	return DefWindowProc (hwnd, nMsg, wParam, lParam);
}

HWND fwhdemo_createMsgWindow(HINSTANCE hInst, int nShow) {
	HWND         hwndMain;        /* Handle for the main window. */
    WNDCLASSEX   wndclass;        /* A window class structure. */
    char*        fwMainWndClassName = "FWHID Hand Motion Receiver";/* The name of the main window class */	

    memset (&wndclass, 0, sizeof(WNDCLASSEX));
    wndclass.lpszClassName = fwMainWndClassName;
    wndclass.cbSize = sizeof(WNDCLASSEX);
    wndclass.style = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc = fwhdemo_MsgOnlyWndProc;
    wndclass.hInstance = hInst;
    wndclass.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wndclass.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wndclass.hCursor = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    RegisterClassEx (&wndclass);
    hwndMain = CreateWindow (
            fwMainWndClassName,             /* Class name */
            "Got any Hand Events Lately?",                    /* Caption */
            WS_MINIMIZEBOX | WS_OVERLAPPED | WS_CAPTION |WS_SYSMENU,        /* Style */
            100,//CW_USEDEFAULT,              /* Initial x (use default) */
            100,//CW_USEDEFAULT,              /* Initial y (use default) */
            320,//CW_USEDEFAULT,              /* Initial x size (use default) */
            350,//CW_USEDEFAULT,              /* Initial y size (use default) */
            NULL,                       /* No parent window */
            NULL,                       /* No menu */
            hInst,                      /* This program instance */
            NULL                        /* Creation parameters */
            );
	if(!hwndMain)
		return NULL;

	ShowWindow (hwndMain, nShow);
    UpdateWindow (hwndMain);
	return hwndMain;
}


int APIENTRY WinMain (HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmd, int nShow) {		 
    MSG          msg;             /* A Win32 message structure. */
	BOOL bRet;
		
	gHwndMain = fwhdemo_createMsgWindow(hInst, nShow);

	//FWHID_setHandEventCallback(fwhdemo_displayHandMotion);
	FWHID_setDeviceCreationCallback(fwhdemo_MultiTouchDeviceCreated);	
	FWHID_setDeviceDisposalCallback(fwhdemo_RemovingMultiTouchDevice);	
	FWHID_registerMultiTouchDevices(gHwndMain);
	while( (bRet = GetMessage( &msg, NULL, 0, 0 )) != 0) { 
		if (bRet == -1) {
			// handle the error and possibly exit
		} else {
			TranslateMessage(&msg); 
			DispatchMessage(&msg); 
		}
	}

    return (int)msg.wParam;
}
