/*
 *  HandMotionTest.c
 *  FingerWorks HID Hand Motion SDK (Windows XP Demonstration)
 *
 *  Created byHai Zhu  Sept 29 2003.
 *  Last Updated by Wayne Westerman on Dec 12 2003.
 *  Copyright (c) 2003 FingerWorks, Inc. All rights reserved.
 **/
 /* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is HandMotionTest.c,
 *   Windows XP Demonstration of FingerWorks HID Hand Motion SDK.
 *
 * The Initial Developer of the Original Code is
 * Wayne Westerman, FingerWorks, Inc.
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * by FingerWorks Inc. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
  #if !defined(_WIN32_WINNT) && !defined(WINDRIVER)
#define _WIN32_WINNT 0x0501   // Identify this as a Windows XP application.  This is necessary
                              //    to pull in RawInput (which is an XP exclusive)
  #endif //_WIN32_WINNT

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "FWHID_HandMotion.h"

static HWND gHwndMain = NULL; //global handle for the main msg window

char hidheaderstr[256];
char hiddatastr[256];
char hiddevicestr[256];
//char hidrawstr[256];
void fwhdemo_PaintHIDinfo(HWND hWnd, HDC hdc, LPPAINTSTRUCT lpPS) {
	RECT rc;

	GetClientRect(hWnd, &rc);
	DrawText(hdc, hiddevicestr, (int)strlen(hiddevicestr), &rc, DT_CENTER);
	OffsetRect(&rc,0,50); // move the draw position down a bit
	DrawText(hdc, hidheaderstr, (int)strlen(hidheaderstr), &rc, DT_CENTER);
	OffsetRect(&rc,0,100); // move the draw position down a bit
	DrawText(hdc, hiddatastr, (int)strlen(hiddatastr), &rc, DT_CENTER);
	OffsetRect(&rc,0,300);
	//OffsetRect(&rc,0,400);
	//DrawText(hdc, hidrawstr, strlen(hidrawstr), &rc, DT_CENTER);

}


void fwhdemo_displayHandMotion(int handEventID, FWHID_HandMotionPtr pHandMotion) {
	char debugbuf[256];
	char *bimanualname;
	char *handname;
	char *handeventname;
    char *handevent_typenames[] = {
        "NullHandEvent",
        "ChordSelect",
        "ChordRelease",
        "...reserved...",
        "ActionCommand",
        "...reserved...",
        "...reserved...",
        "...reserved...",
        "Pressure",
        "Translate",
        "Rotate",
        "Translate+Rotate",
        "Scale",
        "Translate+Scale",
        "Rotate+Scale",
        "Motion"
    };

	FWHID_snprintfHandMotion(debugbuf,256,handEventID,pHandMotion);
	OutputDebugString(debugbuf);

    if(pHandMotion->selectedChord.flags.bimanualSync) {
        bimanualname = "Bimanual ";
    } else bimanualname = "";

    if(pHandMotion->selectedChord.flags.lefthand) {
        handname = "LeftHand  ";
    } else if(pHandMotion->selectedChord.flags.righthand) {
        handname = "RightHand ";
    } else handname = "NullHand  ";

    if(handEventID >= 0 && handEventID < kEventHandIDmax) {
        handeventname = handevent_typenames[handEventID];
    } else handeventname = "NullEvent ";

    wsprintf(hidheaderstr,"%s%s %s: \nSchord=0x%04x Czone=0x%04x Axes=0x%02x\n Action=0x%02x Pressure=%04u \nX=%d Y=%d Z=%d Rz=%d\n",
			bimanualname, handname,handeventname,
           pHandMotion->selectedChord.word, pHandMotion->currentFingers.word,pHandMotion->slideAxesCode,
		   pHandMotion->handActionCode, pHandMotion->handPressure,
           pHandMotion->Xtrans,pHandMotion->Ytrans,pHandMotion->Ztrans,pHandMotion->Zrot);
	InvalidateRect(gHwndMain,0,TRUE);
	SendMessage(gHwndMain,WM_PAINT,0,0);

}





LRESULT CALLBACK fwhdemo_MsgOnlyWndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lParam) {
    PAINTSTRUCT   ps;              /* Also used during window drawing */
	HDC hdc;

    switch (nMsg) {
        case WM_DESTROY:
			FWHID_releaseMultiTouchDevices();
            PostQuitMessage (0);
            return 0;

        case WM_PAINT:
			hdc = BeginPaint(hwnd, &ps);
			fwhdemo_PaintHIDinfo(hwnd, hdc, &ps);
			EndPaint(hwnd, &ps);
			return 0;
		//case WM_ERASEBKGND:
		//	return (LRESULT)1; // Say we handled it.
	}

	//IMPORTANT: let Hand Motion library check/process WM_INPUT or WM_DEVICECHANGED messages
	if(FWHID_WndProc (hwnd, nMsg, wParam, lParam))
		return TRUE;

    //if none of above pass to Default Window procedure
	return DefWindowProc (hwnd, nMsg, wParam, lParam);
}

HWND fwhdemo_createMsgWindow(HINSTANCE hInst, int nShow) {
	HWND         hwndMain;        /* Handle for the main window. */
    WNDCLASSEX   wndclass;        /* A window class structure. */
    char*        fwMainWndClassName = "FWHID Hand Motion Receiver";/* The name of the main window class */

    memset (&wndclass, 0, sizeof(WNDCLASSEX));
    wndclass.lpszClassName = fwMainWndClassName;
    wndclass.cbSize = sizeof(WNDCLASSEX);
    wndclass.style = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc = fwhdemo_MsgOnlyWndProc;
    wndclass.hInstance = hInst;
    wndclass.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wndclass.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wndclass.hCursor = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    RegisterClassEx (&wndclass);
    hwndMain = CreateWindow (
            fwMainWndClassName,             /* Class name */
            "Got any Hand Events Lately?",                    /* Caption */
            WS_MINIMIZEBOX | WS_OVERLAPPED | WS_CAPTION |WS_SYSMENU,        /* Style */
            100,//CW_USEDEFAULT,              /* Initial x (use default) */
            100,//CW_USEDEFAULT,              /* Initial y (use default) */
            320,//CW_USEDEFAULT,              /* Initial x size (use default) */
            350,//CW_USEDEFAULT,              /* Initial y size (use default) */
            NULL,                       /* No parent window */
            NULL,                       /* No menu */
            hInst,                      /* This program instance */
            NULL                        /* Creation parameters */
            );
	if(!hwndMain)
		return NULL;

	ShowWindow (hwndMain, nShow);
    UpdateWindow (hwndMain);
	return hwndMain;
}


int APIENTRY
WinMain (HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmd, int nShow) {

    MSG          msg;             /* A Win32 message structure. */
	BOOL bRet;

	gHwndMain = fwhdemo_createMsgWindow(hInst, nShow);

	FWHID_registerMultiTouchDevices(gHwndMain);
	FWHID_setHandEventCallback(fwhdemo_displayHandMotion);
	//FWHID_setGamepadEventCallback(fwhdemo_displayGamepadMotion);


	while( (bRet = GetMessage( &msg, NULL, 0, 0 )) != 0) {
		if (bRet == -1) {
			// handle the error and possibly exit
		} else {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

    return (int)msg.wParam;
}
