/*
 *  FWHID_HandFingerStats.h
 *  FWHID Utilities -- External API Declarations
 *
 *  Created by Wayne Westerman on Wed Jun 25 2003.
 *  Last Modified by Wayne Westerman on Mon Nov 17 2003.
 *  Copyright (c) 2003 FingerWorks, Inc. All rights reserved.
 *
	IMPORTANT:  		This FingerWorks software is supplied to you by FingerWorks, Inc.
				("FingerWorks") in consideration of your agreement to the following terms, and your
				use, installation, or modification of this FingerWorks software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install or modify this FingerWorks software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, FingerWorks grants you a personal, non-exclusive license, under FingerWorkss
				copyrights in this original FingerWorks software (the "FingerWorks Software"), to use,
				reproduce, and modify the FingerWorks Software for personal, academic, research, or artistic purposes.
                                You may NOT distribute this FingerWorks Software, with or
                                without modifications, in either source or binary form.

                                Neither the name, trademarks, service marks or logos of
				FingerWorks, Inc. may be used to endorse or promote products derived from the
				FingerWorks Software without specific prior written permission from FingerWorks.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by FingerWorks herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the FingerWorks
				Software may be incorporated.

	Disclaimer:		The FingerWorks Software is provided by FingerWorks on an "AS IS" basis.  FINGERWORKS MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE FINGERWORKS SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL FINGERWORKS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE FINGERWORKS SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF FINGERWORKS HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _FWHID_FingerStats_h_
#define _FWHID_FingerStats_h_

#include "FWHID_Path.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct fwhid_fingerstats_t {
    FWHIDfingerID finger_id;
    int assigned_path_id;
    char *name;
    //?
} FWHID_FingerStats, *FWHID_FingerStatsPtr;


typedef struct fwhid_handstats_t {
    FWHIDhandID hand_id;
    char *name;
    int num_contacts; //includes all contact types-- should be = num_fingers + num_palm_contacts
    int num_palm_contacts; //includes palm heels and forepalms
    int num_fingers; //count includes touching thumb, index, middle, ring, pinky
    int last_num_fingers; //num_fingers from previous frame
    int max_fingers; //max count since num_fingers has been > 0
    int num_fingers_proper; //count includes touching index, middle, ring, pinky, but NOT thumb
    int last_num_fingers_proper; //num_fingers_proper from previous frame
    long tfirst_fingerpress_ms; //device milliseconds
    long tlast_fingerpress_ms; //device milliseconds
    long tfirst_fingerrelease_ms; //device milliseconds
    long tlast_fingerrelease_ms; //device milliseconds
    FWHID_FingerStats fingers[kFWHID_MAX_CONTACTS_PER_HAND]; //index by finger_id to lookup path_id (will be kFWHID_NULL_PATH_ID if finger not touching).
} FWHID_HandStats, *FWHID_HandStatsPtr;



//print statistics on device and contact data
extern void FWHID_printHandStats(FWMultiTouchDevicePtr fwdevice,FWHID_HandStatsPtr pHand,
                         int nesting_level);
extern void FWHID_printFrameStats(FWMultiTouchDevicePtr fwdevice,int nesting_level);


//accessing path, finger, and hand statistics data structures
extern FWHID_HandStatsPtr FWHID_getLeftHandStats(FWMultiTouchDevicePtr fwdevice);
extern FWHID_HandStatsPtr FWHID_getRightHandStats(FWMultiTouchDevicePtr fwdevice);
extern FWHID_PathPtr FWHID_getIdentifiedPath(FWMultiTouchDevicePtr fwdevice, FWHIDhandID hand_id, FWHIDfingerID finger_id);
extern FWHID_FingerStatsPtr FWHID_getFingerStats(FWMultiTouchDevicePtr fwdevice, FWHIDhandID hand_id, FWHIDfingerID finger_id);

#ifdef __cplusplus
}
#endif

#endif // _FWHID_HandFingerStats.h

