/*
 *  fwhid_DeviceInternals.h
 *  FWHID Utilities -- Internal WinXP implementation of contact frame HID parser.
 *
 *  Created by Wayne Westerman on Wed Jun 25 2003.
 *  Last Modified by James Orr on Mon Aug 23 2004.
 *  Copyright (c) 2003 FingerWorks, Inc. All rights reserved.
 *
	IMPORTANT:  		This FingerWorks software is supplied to you by FingerWorks, Inc.
				("FingerWorks") in consideration of your agreement to the following terms, and your
				use, installation, or modification of this FingerWorks software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install or modify this FingerWorks software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, FingerWorks grants you a personal, non-exclusive license, under FingerWorkss
				copyrights in this original FingerWorks software (the "FingerWorks Software"), to use,
				reproduce, and modify the FingerWorks Software for personal, academic, research, or artistic purposes.
                                You may NOT distribute this FingerWorks Software, with or
                                without modifications, in either source or binary form.

                                Neither the name, trademarks, service marks or logos of
				FingerWorks, Inc. may be used to endorse or promote products derived from the
				FingerWorks Software without specific prior written permission from FingerWorks.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by FingerWorks herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the FingerWorks
				Software may be incorporated.

	Disclaimer:		The FingerWorks Software is provided by FingerWorks on an "AS IS" basis.  FINGERWORKS MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE FINGERWORKS SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL FINGERWORKS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE FINGERWORKS SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF FINGERWORKS HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _fwhid_deviceInternals_h_
#define _fwhid_deviceInternals_h_

#include "FWHID_MultiTouchDevice.h"
 #if defined(__APPLE__)
#include "macosx_HandMotionInternals.h"
 #endif //__APPLE__	


//#define ENABLE_DEBUG_OUTPUT

#ifdef __cplusplus
extern "C" {
#endif

#define kFWHID_FINGERWORKS_USB_VENDOR_ID 0xe97
#define kFWHIDUsage_Dig_FingerID 0x48
#define kFWHIDUsage_Dig_HandID 0x49
#define kFWHIDUsage_Dig_FrameNo 0x4A
#define kFWHIDUsage_Dig_Timestamp 0x4B
#define kFWHIDUsage_Dig_Xabsolute 0x50
#define kFWHIDUsage_Dig_Yabsolute 0x51
#define kFWHIDUsage_Dig_Zabsolute 0x52
#define kFWHIDUsage_Dig_Xvel 0x60
#define kFWHIDUsage_Dig_Yvel 0x61
#define kFWHIDUsage_Dig_Zvel 0x62


#define	kFWHID_Path_ReportID 0x10
#define	kFWHID_ProximityImage_ReportID 0x4
#define	kFWHID_ImagerFeatures_ReportID 0x1

#define kFWHID_PackbitsDataWords 15

#define kFWHIDPage_ProximityImages 0x24

#define kFWHIDUsage_PI_DataWord0 0x0
#define kFWHIDUsage_PI_DataWord1 0x1
#define kFWHIDUsage_PI_DataWord14 0xE
#define kFWHIDUsage_PI_DataWord15 0xF
#define kFWHIDUsage_PI_DataWord16 0x10

#define kFWHIDUsage_PI_PackbitsReport 0xE0
#define kFWHIDUsage_PI_FrameNum 0xF0
#define kFWHIDUsage_PI_SubpacketNum 0xF1

#define	kFW_FlathandResetMask 0x1
#define	kFW_GestureProcessingMask 0x2

typedef struct {
    FWBYTE reportID;
    FWBYTE sensor_rows;
    FWBYTE sensor_columns;
    FWBYTE stream_enables;
    FWBYTE processing_enables; //gesture_processing, LSB flathand_reset?
    FWBYTE frames2skip; //image in lower nibble, path in upper nibble
	FWBYTE noise_tolerance;
} FWHID_ImagerFeatures;

typedef char FWHID_Str255[256];

typedef struct fw_multitouchdevice_t {
    long device_time_ms; //milliseconds
    FWHID_AbsoluteTime host_time; //milliseconds

    long usb_vendorID;
    long usb_productID;
    long usb_firmware_version;
    FWHID_Str255 usb_manufacturer;
    FWHID_Str255 usb_product;
    FWHID_Str255 usb_serial;
    long usb_locID;
    long keymatrix_version;

    //looked up or reported by device, for computing surface_dimensions_cm
    int sensor_rows;
    int sensor_columns;
    double origin_row;
    double origin_column;
    double sensor_row_pitch;
    double sensor_column_pitch;

    FWSurfaceRectangle surface_dimensions_cm;
    FWHID_ImagerFeatures imager_features;

	FWHID_ContactFrame partial_frame;
	FWHID_ContactFrame complete_frame;

    FWHID_Path persistent_paths[kFWHID_MAX_CONTACT_PATHS];
#if COMPILE_HANDFINGER_STATS
    FWHID_HandStats lefthand_stats;
    FWHID_HandStats righthand_stats;
#endif //COMPILE_HANDFINGER_STATS
#if defined(WIN32)
	void *pHidDeviceInfo;
#endif //WIN32
#if defined(__APPLE__)
	FWHID_HandMotionReconstruction leftHandState;
	FWHID_HandMotionReconstruction rightHandState;
#endif //__APPLE__	

    void (*gotContact_callback)(struct fw_multitouchdevice_t *fwdevice, void* targetData);
    void (*gotContactFrame_callback)(struct fw_multitouchdevice_t *fwdevice, void* targetData);
    void (*gotPathFrame_callback)(struct fw_multitouchdevice_t *fwdevice, void* targetData);
    //void (*gotImageFrame_callback)(struct fw_multitouchdevice_t *fwdevice, void* targetData);
    //void (*gotHandMotion_callback)(struct fw_multitouchdevice_t *fwdevice, void* targetData);
} FWMultiTouchDevice;



extern void fwhid_printIndent(int nesting_level);

//implemented in FWHID_Contact.c
extern void fwhid_initContact(FWHID_ContactPtr pContact);
extern void fwhid_processRawContact(FWMultiTouchDevicePtr fwdevice,FWBYTE *bytearray, int numbytes, int firmware_version);
extern void fwhid_processContact(FWMultiTouchDevicePtr fwdevice, FWHID_ContactPtr pContact);
extern void fwhid_processContactFrame(FWMultiTouchDevicePtr fwdevice);

//implemented in FWHID_Path.c
extern void fwhid_initPath(FWHID_PathPtr pPath, int path_id);
extern void fwhid_updatePaths(FWMultiTouchDevicePtr fwdevice);

 #if COMPILE_HANDFINGER_STATS
//implemented in FWHID_HandFingerStats.c
extern void fwhid_initHandStats(FWHID_HandStatsPtr pHand, FWHIDhandID hand_id);
extern void fwhid_updateHandStats(FWMultiTouchDevicePtr fwdevice, FWHID_HandStatsPtr pHand);
 #endif //COMPILE_HANDFINGER_STATS

//implemented in FWHID_MultiTouchDevice.c
extern unsigned fw_getFeatureReport(FWMultiTouchDevicePtr fwdevice, const unsigned reportID, 		
                                    void* reportBuffer, unsigned long reportBufferSize);
extern unsigned fw_setFeatureReport(FWMultiTouchDevicePtr fwdevice, const unsigned reportID, 		
                                    void* reportBuffer, unsigned long reportBufferSize);

extern void fw_disposeMultiTouchDevice(FWMultiTouchDevicePtr fwdevice);

extern void fw_notifyMultiTouchDeviceComplete(FWMultiTouchDevicePtr fwdevice);
extern FWMultiTouchDevicePtr fw_createMultiTouchDevice(long vendorID,long productID,
                                          long version,char* manufacturer,
                                          char* product,char* serial,long locID);



#ifdef __cplusplus
}
#endif

#endif // _fwhid_deviceInternals_h_

