/*
 *  fwhid_HandMotionInternals.h
 *  FingerWorks HID Hand Motion SDK (Library Internal Declarations)
 *
 *  Created by Wayne Westerman on Wed Jun 25 2003.
  *  Last Updated by Wayne Westerman on Wed Nov 19 2003.
  *  Copyright (c) 2003 FingerWorks, Inc. All rights reserved.
  **/
 /* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is fwhid_HandMotionInternals.h,
 *   External API for FingerWorks HID Hand Motion SDK.
 *
 * The Initial Developer of the Original Code is
 * Wayne Westerman, FingerWorks, Inc.
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * by FingerWorks Inc. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


#include "FWHID_HandMotion.h"

#ifndef _fwhid_HandMotionInternals_h_
#define _fwhid_HandMotionInternals_h_

#ifdef __cplusplus
extern "C" {
#endif



//private API needed by platform-specific library internals.  DO NOT USE!--------------
void fwhand_clearMotion(FWHID_HandMotionPtr pHandMotion);
int fwhand_getMotionType(FWHID_HandMotionPtr pHandMotion);
void fwhand_processAccumulatedMotion(FWHID_HandMotionPtr pHandMotion);
void fwhand_processChordChange(FWHID_HandMotionPtr pHandMotion, unsigned new_chord_word);
void fwhand_parseRawHandMotion(FWBYTE *bytearray, int numbytes, int firmware_version);
void fwhand_clearState(void);



/* USB HID Usages needed by platform-specific library internals. */
 #ifndef kFWHID_FINGERWORKS_USB_VENDOR_ID
#define kFWHID_FINGERWORKS_USB_VENDOR_ID 0xe97
 #endif //kFWHID_FINGERWORKS_USB_VENDOR_ID
#define kFWHID_FINGERWORKS_HANDMOTION_INTERFACE 3

#define kFWHIDPage_GenericDesktop 0x1
#define kFWHIDPage_Digitizer 0xD
#define kFWHIDPage_AlphanumericDisplay 0x14
 #ifndef kFWHIDPage_ProximityImages
#define kFWHIDPage_ProximityImages 0x24
 #endif //kFWHIDPage_ProximityImages

#define kFWHIDUsage_GD_Mouse 0x2
#define kFWHIDUsage_GD_Joystick 0x4
#define kFWHIDUsage_GD_GamePad 0x5
#define kFWHIDUsage_GD_MultiAxisController 0x8
#define kFWHIDUsage_Dig_MultiPointDigitizer 0xC
#define kFWHIDUsage_Dig_ProximityImager 0x10

#define kFWHIDUsage_GD_RightHandMotion 0xE
#define kFWHIDUsage_GD_LeftHandMotion 0xF

#define kFWHIDUsage_GD_Sx 0x49
#define kFWHIDUsage_GD_Sy 0x4A
#define kFWHIDUsage_GD_Sz 0x4B

#define kFWHIDUsage_GD_StartChord 0x70
#define kFWHIDUsage_GD_CurrentFingers 0x72

#define kFWHID_RightHandMotion_ReportID 0x8
#define kFWHID_LeftHandMotion_ReportID 0xA

#ifdef __cplusplus
}
#endif

#endif // _fwhid_HandMotionInternals_h_

