/*
 *  macosx_ContactInternals.h
 *  FWHID Utilities -- Internal declarations for MacOSX implementation of contact frame HID parser.
 *
 *  Created by Wayne Westerman on Wed Jun 25 2003.
 *  Last Modified by Wayne Westerman on Mon Nov 17 2003.
 *  Copyright (c) 2003 FingerWorks, Inc. All rights reserved.
 *
	IMPORTANT:  		This FingerWorks software is supplied to you by FingerWorks, Inc.
				("FingerWorks") in consideration of your agreement to the following terms, and your
				use, installation, or modification of this FingerWorks software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install or modify this FingerWorks software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, FingerWorks grants you a personal, non-exclusive license, under FingerWorkss
				copyrights in this original FingerWorks software (the "FingerWorks Software"), to use,
				reproduce, and modify the FingerWorks Software for personal, academic, research, or artistic purposes.
                                You may NOT distribute this FingerWorks Software, with or
                                without modifications, in either source or binary form.

                                Neither the name, trademarks, service marks or logos of
				FingerWorks, Inc. may be used to endorse or promote products derived from the
				FingerWorks Software without specific prior written permission from FingerWorks.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by FingerWorks herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the FingerWorks
				Software may be incorporated.

	Disclaimer:		The FingerWorks Software is provided by FingerWorks on an "AS IS" basis.  FINGERWORKS MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE FINGERWORKS SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL FINGERWORKS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE FINGERWORKS SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF FINGERWORKS HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _macosx_ContactInternals_h_
#define _macosx_ContactInternals_h_


#include <IOKit/IOTypes.h>
#include <IOKit/hid/IOHIDUsageTables.h>

#include <IOKit/HID/IOHIDLib.h>

#include "FWHID_MultiTouchDevice.h"
#include "fwhid_DeviceInternals.h"
#include "HID_Utilities_External.h"

#ifdef __cplusplus
extern "C" {
#endif


typedef struct {
    IOHIDElementCookie gTimestamp;
    IOHIDElementCookie gFrameNo;
    IOHIDElementCookie gPathID;
    IOHIDElementCookie gFingerID;
    IOHIDElementCookie gHandID;
    IOHIDElementCookie gTouch;
    IOHIDElementCookie gUntouch;
    IOHIDElementCookie gXPos;
    IOHIDElementCookie gYPos;
    IOHIDElementCookie gXVel;
    IOHIDElementCookie gYVel;
    IOHIDElementCookie gProximity;
    IOHIDElementCookie gOrientation;
    IOHIDElementCookie gEccentricity;
} FWHID_ContactCookies;

typedef struct {
    pRecElement gTimestamp;
    pRecElement gFrameNo;
    pRecElement gPathID;
    pRecElement gFingerID;
    pRecElement gHandID;
    pRecElement gTouch;
    pRecElement gUntouch;
    pRecElement gXPos;
    pRecElement gYPos;
    pRecElement gXVel;
    pRecElement gYVel;
    pRecElement gProximity;
    pRecElement gOrientation;
    pRecElement gEccentricity;
} FWHID_ContactElements;

typedef struct {
    IOHIDElementCookie frameNum;
    IOHIDElementCookie subpacketNum;
    IOHIDElementCookie dataWords[kFWHID_PackbitsDataWords];
} fwhid_ProximityImageElements;

typedef struct {
    pRecElement frameNum;
    pRecElement subpacketNum;
    pRecElement dataWords[kFWHID_PackbitsDataWords];
} fwhid_ProximityImageCookies;

typedef struct {
    fwhid_ProximityImageCookies image_cookies;
    fwhid_ProximityImageElements image_elements;
    int frame_num;
    int subpacket_num;
    Boolean gotFrameNum_change;
    Boolean gotSubpacketNum_change;
    long device_timestamp;
    int expected_bytecount;
    unsigned dataWords[kFWHID_PackbitsDataWords];
} fwhid_ImageReconstructionState, *fwhid_ImageReconstructionStatePtr;

typedef struct {
    FWHID_ContactCookies cookies;
    FWHID_ContactElements elements;
    FWHID_Contact accumulator;
    Boolean gotFrameNum_change;
    IOHIDElementCookie last_cookie;
    fwhid_ImageReconstructionState image_state;
} fwhid_ContactReconstructionState, *fwhid_ContactReconstructionStatePtr;

extern fwhid_ContactReconstructionStatePtr fwhid_createContactReconstructionState(void);
extern void fwhid_disposeContactReconstructionState(fwhid_ContactReconstructionStatePtr pContactState);
extern void fwhid_updateContactAccumulator(pRecDevice pHIDDevice,
									IOHIDEventStruct *pHIDEvent);
extern void fwhand_storeContactPathCookies (pRecDevice pHIDDevice);

#ifdef __cplusplus
}
#endif

#endif // _macosx_ContactInternals.h

