/*
 *  macosx_HandMotionInternals.h
 *  FingerWorks HID Hand Motion SDK (Mac OS X HID parser definitions)
 *
 *  Created by Wayne Westerman on Wed Jun 25 2003.
 *  Last Updated by Wayne Westerman on Friday Feb 20 2004.
 *  Copyright (c) 2003-2004 FingerWorks, Inc. All rights reserved.
 **/
/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is macosx_HandMotionInternals.h,
*   Mac OS X HID parser definitions for FingerWorks HID Hand Motion SDK.
*
* The Initial Developer of the Original Code is
* Wayne Westerman, FingerWorks, Inc.
*
* Portions created by the Initial Developer are Copyright (C) 2003-2004
* by FingerWorks, Inc. All Rights Reserved.
*
* Contributor(s):
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

#ifndef _macosx_HandMotionInternals_h_
#define _macosx_HandMotionInternals_h_

#include <IOKit/IOTypes.h>
#include <IOKit/hid/IOHIDUsageTables.h>

#include <IOKit/HID/IOHIDLib.h>

#include "FWHID_HandMotion.h"
#include "fwhid_HandMotionInternals.h"
#include "HID_Utilities_External.h"

#ifdef __cplusplus
extern "C" {
#endif



typedef struct {
    IOHIDElementCookie StartChord;
    IOHIDElementCookie CurrentFingers;
    IOHIDElementCookie HandPressure;
    IOHIDElementCookie HandPositionX;
    IOHIDElementCookie HandPositionY;
    IOHIDElementCookie x;
    IOHIDElementCookie y;
    IOHIDElementCookie z;
    IOHIDElementCookie Rx;
    IOHIDElementCookie Ry;
    IOHIDElementCookie Rz;
    IOHIDElementCookie Sx;
    IOHIDElementCookie Sy;
    IOHIDElementCookie Sz;
	//in v <= 1.40, HandActionCode will actually be fou in CurrentFingers word element
	//in v >= 1.41, SlideAxesCode will actually be found in CurrentFingers word element
    IOHIDElementCookie SlideAxesCode;
    IOHIDElementCookie HandActionCode;
} FWHID_HandMotionCookies;

typedef struct {
    pRecElement StartChord;
    pRecElement CurrentFingers;
    pRecElement HandPressure;
    pRecElement HandPositionX;
    pRecElement HandPositionY;
    pRecElement x;
    pRecElement y;
    pRecElement z;
    pRecElement Rx;
    pRecElement Ry;
    pRecElement Rz;
    pRecElement Sx;
    pRecElement Sy;
    pRecElement Sz;
	//in v <= 1.40, HandActionCode will actually be fou in CurrentFingers word element
	//in v >= 1.41, SlideAxesCode will actually be found in CurrentFingers word element
    pRecElement SlideAxesCode;
    pRecElement HandActionCode;
} FWHID_HandMotionElements;


typedef struct {
    IOHIDElementCookie min_cookie;
    IOHIDElementCookie max_cookie;
    FWHID_HandMotionCookies cookies;
    FWHID_HandMotionElements elements;
    FWHID_HandMotion motion;
} FWHID_HandMotionReconstruction, *FWHID_HandMotionReconstructionPtr;

extern bool fwhand_handleHIDEvent(IOHIDEventStruct *eventP,FWHID_HandMotionReconstructionPtr pLeftHandState, 
														  FWHID_HandMotionReconstructionPtr pRightHandState);
extern void fwhand_finishHIDEventBatch(FWHID_HandMotionReconstructionPtr pLeftHandState, 
														  FWHID_HandMotionReconstructionPtr pRightHandState);
extern void fwhand_storeHandMotionCookies(pRecDevice pHIDDevice,FWHID_HandMotionReconstructionPtr pLeftHandState, 
														  FWHID_HandMotionReconstructionPtr pRightHandState);


#ifdef __cplusplus
}
#endif

#endif // _macosx_HandMotionInternals_h_

