/*
 *  XWinderCalcs.h
 *  FingerWorks XWinder window manipulation utility 
 *    Platform-independent External API declarations
 *
 *  Created by Wayne Westerman on Sunday Nov 9 2003.
 *  Last Updated by Wayne Westerman on Wednesday Feb 11 2004.
 *  Copyright (c) 2003-2004 FingerWorks, Inc. All rights reserved. 
 **/
/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is XWinderCalcs.h,
*   Platform-independent External API declarations for XWinder.
*
* The Initial Developer of the Original Code is
* Wayne Westerman, FingerWorks, Inc.
*
* Portions created by the Initial Developer are Copyright (C) 2003-2004
* by FingerWorks Inc. All Rights Reserved.
*
* Contributor(s):
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */


#ifndef _XWinderCalcs_h_
#define _XWinderCalcs_h_

#include <stdio.h>
#include <math.h>
#include <FWHID_MultiTouchDevice.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef double XWFWAbsoluteTime;
typedef double XWFWTimeInterval;

typedef struct {
    struct {
        float x;
        float y;
    } origin;
    struct {
        float width;
        float height;
    } size;
} XWFWRect;

typedef void (*XWFWMotionOnlyCALLBACK)(FWHID_HandMotionPtr);

/** The central hand motion event handler in XWinderCalcs.c used as a callback from FWHID_HandMotion library */
extern void XWFWhandleHandMotion(int handEventID, FWHID_HandMotionPtr pHandMotion);

/** XWFWhandleHandMotion calls these handlers in response to hand motion events
   their implementation will be platform (window-system) specific in XWinder*Shell.? */
extern void XWFWgrabWindow(FWHID_HandMotionPtr onehandp);
extern void XWFWdropGrabbedWindow(FWHID_HandMotionPtr onehandp);
extern void XWFWchangeGrabbedWindowBounds(XWFWRect *by_amount, FWHID_HandMotionPtr onehandp);
extern void XWFWpanWindow(int byamount);
extern void XWFWhandleActionCode(int action_code);
//extern void XWFWtoggleMaximizeRestore(FWHID_HandMotionPtr onehandp);
//extern void XWFWminimize(FWHID_HandMotionPtr onehandp);
//extern void XWFWsendToBack(FWHID_HandMotionPtr onehandp);
//extern void XWFWbringToTop(FWHID_HandMotionPtr onehandp);

/* timer and delay procedures (platform-specific implementation in XWinder*Shell.? ) */
extern XWFWAbsoluteTime XWFWgetAbsoluteTime(void);
extern void XWFWstopPostponedWindTimer(void);
extern void XWFWstartPostponedWindTimer(XWFWMotionOnlyCALLBACK xwfw_windNowCallback,
										FWHID_HandMotionPtr pendingSourceHandp, XWFWTimeInterval delay);

/* preference query procedures (platform-specific implementation in XWinder*Shell.? ) */
extern bool XWFWgetBooleanPreference(char *preference_name);
extern double XWFWgetDoublePreference(char *preference_name);

#ifdef __cplusplus
}
#endif

#endif //_XWinderCalcs_h_