/*
 File:		InspectorWindow.m

 Description: 	This is the implementation for the InspectorWindow class that handles displaying information about whatever UI element the mouse cursor is over (or whatever element has been locked into focus).
 
 This sample demonstrates the Accessibility API introduced in Mac OS X 10.2.

 Author:		KA, ME, and MCF

 Modified by:
 FingerWorks on Nov. 22nd, 2003 to put AppShell attribute-querying class methods into InspectorWindow class methods

 Copyright: 	 Copyright 2002-2003 Apple Computer, Inc. All rights reserved.

 Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
 ("Apple") in consideration of your agreement to the following terms, and your
 use, installation, modification or redistribution of this Apple software
 constitutes acceptance of these terms.  If you do not agree with these terms,
 please do not use, install, modify or redistribute this Apple software.

 In consideration of your agreement to abide by the following terms, and subject
 to these terms, Apple grants you a personal, non-exclusive license, under Apples
 copyrights in this original Apple software (the "Apple Software"), to use,
 reproduce, modify and redistribute the Apple Software, with or without
 modifications, in source and/or binary forms; provided that if you redistribute
 the Apple Software in its entirety and without modifications, you must retain
 this notice and the following text and disclaimers in all such redistributions of
 the Apple Software.  Neither the name, trademarks, service marks or logos of
 Apple Computer, Inc. may be used to endorse or promote products derived from the
 Apple Software without specific prior written permission from Apple.  Except as
 expressly stated in this notice, no other rights or licenses, express or implied,
 are granted by Apple herein, including but not limited to any patent rights that
 may be infringed by your derivative works or by other works in which the Apple
 Software may be incorporated.

 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
 WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
 COMBINATION WITH YOUR PRODUCTS.

 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
                        GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
 OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
 (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */


#import "XWinder_Prefix.pch"
#import "InspectorWindow.h"

@implementation InspectorWindow

//class method from in AppShell.m in Apple's original UIElementInspector sample
+ (NSString *)descriptionOfValue:(CFTypeRef)theValue beingVerbose:(BOOL)beVerbose
{
    NSString *	theValueDescString	= NULL;
    
    if (theValue) {
        
        if (AXValueGetType(theValue) != kAXValueIllegalType) {
            theValueDescString = [InspectorWindow stringDescriptionOfAXValue:theValue beingVerbose:beVerbose];
        }
        else if (CFGetTypeID(theValue) == CFArrayGetTypeID()) {
            theValueDescString = [NSString stringWithFormat:@"<array of size %d>", [(NSArray *)theValue count]];
        }
        else if (CFGetTypeID(theValue) == AXUIElementGetTypeID()) {
            
            NSString *	uiElementRole  	= NULL;
            
            if (AXUIElementCopyAttributeValue( (AXUIElementRef)theValue, kAXRoleAttribute, (CFTypeRef *)&uiElementRole ) == kAXErrorSuccess) {
                NSString *	uiElementTitle  = NULL;
                
                uiElementTitle = [InspectorWindow valueOfExistingAttribute:kAXTitleAttribute ofUIElement:(AXUIElementRef)theValue];
                
                if (uiElementTitle != nil) {
                    theValueDescString = [NSString stringWithFormat:@"<%@: %@>", uiElementRole, uiElementTitle];
                }
                else {
                    theValueDescString = [NSString stringWithFormat:@"<%@>", uiElementRole];
                }
                [uiElementRole release];
            }
            else {
                theValueDescString = [(id)theValue description];
            }
        }
        else {
            theValueDescString = [(id)theValue description];
        }
    }
    
    return theValueDescString;
}


//class method from in AppShell.m in Apple's original UIElementInspector sample
+ (NSString *)descriptionForUIElement:(AXUIElementRef)uiElement attribute:(NSString *)name beingVerbose:(BOOL)beVerbose
{
    NSString *	theValueDescString	= NULL;
    CFTypeRef	theValue;
    CFIndex	count;
    if (([name isEqualToString:NSAccessibilityChildrenAttribute]
         ||
         [name isEqualToString:NSAccessibilityRowsAttribute]
         )
        &&
        AXUIElementGetAttributeValueCount(uiElement, (CFStringRef)name, &count) == kAXErrorSuccess) {
        // No need to get the value of large arrays - we just display their size. We don't want to do this with every attribute because AXUIElementGetAttributeValueCount on non-array valued attributes will cause debug spewage.
        theValueDescString = [NSString stringWithFormat:@"<array of size %d>", count];
    } else if (AXUIElementCopyAttributeValue ( uiElement, (CFStringRef)name, &theValue ) == kAXErrorSuccess && theValue) {
        theValueDescString = [self descriptionOfValue:theValue beingVerbose:beVerbose];
    }
    return theValueDescString;
}

//class method from in AppShell.m in Apple's original UIElementInspector sample
+ (id)valueOfExistingAttribute:(CFStringRef)attribute ofUIElement:(AXUIElementRef)element
{
    id result = nil;
    NSArray *attrNames;
    
    if (AXUIElementCopyAttributeNames(element, (CFArrayRef *)&attrNames) == kAXErrorSuccess) {
        if ( [attrNames indexOfObject:(NSString *)attribute] != NSNotFound
             &&
             AXUIElementCopyAttributeValue(element, attribute, (CFTypeRef *)&result) == kAXErrorSuccess
             ) {
            [result autorelease];
        }
        [attrNames release];
    }
    return result;
}

//class method from in AppShell.m in Apple's original UIElementInspector sample
+ (NSString *)stringDescriptionOfAXValue:(CFTypeRef)valueRef beingVerbose:(BOOL)beVerbose
{
    NSString *result = @"AXValue???";
    
    switch (AXValueGetType(valueRef)) {
        case kAXValueCGPointType: {
            CGPoint point;
            if (AXValueGetValue(valueRef, kAXValueCGPointType, &point)) {
                if (beVerbose)
                    result = [NSString stringWithFormat:@"<AXPointValue x=%g y=%g>", point.x, point.y];
                else
                    result = [NSString stringWithFormat:@"x=%g y=%g", point.x, point.y];
            }
            break;
        }
        case kAXValueCGSizeType: {
            CGSize size;
            if (AXValueGetValue(valueRef, kAXValueCGSizeType, &size)) {
                if (beVerbose)
                    result = [NSString stringWithFormat:@"<AXSizeValue w=%g h=%g>", size.width, size.height];
                else
                    result = [NSString stringWithFormat:@"w=%g h=%g", size.width, size.height];
            }
            break;
        }
        case kAXValueCGRectType: {
            CGRect rect;
            if (AXValueGetValue(valueRef, kAXValueCGRectType, &rect)) {
                if (beVerbose)
                    result = [NSString stringWithFormat:@"<AXRectValue  x=%g y=%g w=%g h=%g>", rect.origin.x, rect.origin.y, rect.size.width, rect.size.height];
                else
                    result = [NSString stringWithFormat:@"x=%g y=%g w=%g h=%g", rect.origin.x, rect.origin.y, rect.size.width, rect.size.height];
            }
            break;
        }
        case kAXValueCFRangeType: {
            CFRange range;
            if (AXValueGetValue(valueRef, kAXValueCFRangeType, &range)) {
                if (beVerbose)
                    result = [NSString stringWithFormat:@"<AXRangeValue pos=%ld len=%ld>", range.location, range.length];
                else
                    result = [NSString stringWithFormat:@"pos=%ld len=%ld", range.location, range.length];
            }
            break;
        }
        default:
            break;
    }
    return result;
}


- (void)awakeFromNib
{
    // In this example we're using Cocoa's mouseMoved: message to trigger udpating
	//[self setAcceptsMouseMovedEvents:true];
    
}

- (void)mouseMoved:(NSEvent *)theEvent
{
    // Tell AppShell that the mouse moved
	//[AppShell updateCurrentUIElement];
}

- (void)displayInfoForUIElement:(AXUIElementRef)uiElement
{
    [_consoleView setString:[self stringDescriptionOfUIElement:uiElement]];
    [_consoleView display];
}

- (NSArray *)lineageOfUIElement:(AXUIElementRef)element{
    NSArray *lineage = [NSArray array];
    NSString *elementDescr = [InspectorWindow descriptionOfValue:element beingVerbose:NO];
    AXUIElementRef parent = (AXUIElementRef)[InspectorWindow valueOfExistingAttribute:kAXParentAttribute ofUIElement:element];

    if (parent != NULL) {
        lineage = [self lineageOfUIElement:parent];
    }
    return [lineage arrayByAddingObject:elementDescr];
}

- (NSString *)lineageDescriptionOfUIElement:(AXUIElementRef)element {
    NSMutableString *result = [NSMutableString string];
    NSMutableString *indent = [NSMutableString string];
    NSArray *lineage = [self lineageOfUIElement:element];
    NSString *ancestor;
    NSEnumerator *e = [lineage objectEnumerator];
    while (ancestor = [e nextObject]) {
        [result appendFormat:@"%@%@\n", indent, ancestor];
        [indent appendString:@" "];
    }
    return result;
}

- (NSString *)stringDescriptionOfUIElement:(AXUIElementRef)inElement
{
    NSMutableString * 	theDescriptionStr = [[NSMutableString new] autorelease];
    NSArray *		theNames;
    CFIndex			nameIndex;
    CFIndex			numOfNames;

    [theDescriptionStr appendFormat:@"%@", [self lineageDescriptionOfUIElement:inElement]];
    
    //
    // Display attributes
    //
    AXUIElementCopyAttributeNames( inElement, (CFArrayRef *)&theNames );
    if (theNames) {
    
        numOfNames = [theNames count];
        
        if (numOfNames)
            [theDescriptionStr appendString:@"\nAttributes:\n"];

        for( nameIndex = 0; nameIndex < numOfNames; nameIndex++ ) {
            
            NSString *	theName = NULL;
            //id		theValue = NULL;
            Boolean	theSettableFlag = false;
            
            // Grab name
            theName = [theNames objectAtIndex:nameIndex];
                
            // Grab settable field
          	AXUIElementIsAttributeSettable( inElement, (CFStringRef)theName, &theSettableFlag );
            
            // Add string        
            [theDescriptionStr appendFormat:@"   %@%@:  %@\n", theName, (theSettableFlag?@" (W)":@""), [InspectorWindow descriptionForUIElement:inElement attribute:theName beingVerbose:false]];
        
            //[theValue release];
        }
    
        [theNames release];
    }
    
    //
    // Display actions
    //
       	AXUIElementCopyActionNames( inElement, (CFArrayRef *)&theNames );
    if (theNames) {
    
        numOfNames = [theNames count];
        
        if (numOfNames)
            [theDescriptionStr appendString:@"\nActions:\n"];

        for( nameIndex = 0; nameIndex < numOfNames; nameIndex++ ) {
            
            NSString *	theName 		= NULL;
           	NSString *	theDesc 		= NULL;
            
            // Grab name
            theName = [theNames objectAtIndex:nameIndex];
            
            // Grab description
        	AXUIElementCopyActionDescription( inElement, (CFStringRef)theName, (CFStringRef *)&theDesc );
            
            // Add string        
            [theDescriptionStr appendFormat:@"   %@ - %@\n", theName, theDesc];
            
            [theDesc release];
        }
    
        [theNames release];
    }
    
    return theDescriptionStr;
}


- (NSString *)stringDescriptionOfCFArray:(NSArray *)inArray
{

    NSMutableString * 	theDescriptionStr = [[NSMutableString new] autorelease];
    CFIndex				theIndex;
    CFIndex				numOfElements = [inArray count];

    [theDescriptionStr appendFormat:@"{"];
        
    for( theIndex = 0; theIndex < numOfElements; theIndex++ ) {
    
        id theObject = [inArray objectAtIndex:theIndex];
    
    	if (CFGetTypeID(theObject) == CFDictionaryGetTypeID()) {
    
            if (theIndex == 0)
                [theDescriptionStr appendFormat:@"(<UI Element: %d>)", theObject];
            else
                [theDescriptionStr appendFormat:@", (<UI Element: %d>)", theObject];
        }
        else {
            if (theIndex == 0)
                [theDescriptionStr appendFormat:@"%@", [inArray objectAtIndex:theIndex]];
            else
                [theDescriptionStr appendFormat:@", %@", [inArray objectAtIndex:theIndex]];
        }
    }
    
    [theDescriptionStr appendFormat:@"}"];
    
    return theDescriptionStr;
}


- (void)close
{
    [super close];

    [NSApp terminate:NULL];
}

- (void)fontSizeSelected:(id)sender
{
	[_consoleView setFont:[NSFont userFontOfSize:[[sender titleOfSelectedItem] floatValue]]];
    [_consoleView display];
}

- (void)indicateUIElementIsLocked:(BOOL)flag
{
	[_consoleView setTextColor:(flag)?[NSColor redColor]:[NSColor blackColor]];
    [_consoleView display];
}

@end
