/*
 *  XWinderShell.h
 *  FingerWorks XWinder window manipulation utility (Mac OS X Implementation)
 *
 *  Created by Wayne Westerman on Sunday Nov 9 2003.
 *  Last Updated by Wayne Westerman on Wednesday Feb 11 2004.
 *  Copyright (c) 2003-2004 FingerWorks, Inc. All rights reserved.
 **/
/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is XWinderShell.h,
*   Mac OS X Implementation of XWinder window manipulation utility.
*
* The Initial Developer of the Original Code is
* Wayne Westerman, FingerWorks, Inc.
*
* Portions created by the Initial Developer are Copyright (C) 2003
* by FingerWorks Inc. All Rights Reserved.
*
* Contributor(s):
* Scott Menor
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */


#import "XWinder_Prefix.pch"
#import "XWinderCalcs.h"
#import "InspectorWindow.h"

@interface XWinderShell : NSObject {

    IBOutlet InspectorWindow * 		_inspectorWindow;
    NSWindow  *                         _highlightWindow;

    NSStatusItem *                      _statusBarItem;
    NSMenuItem  *                       _itemOrthoStretch;
    NSMenuItem  *                       _itemSizeAspect;
    NSMenu     *                        _statusBarMenu;
    NSRect                              _extendedDesktopBounds;
    NSRect                              _primaryDesktopBounds;
    AXUIElementRef			_grabbedUIWindowRef;

    AXUIElementRef			_systemWideElement;
}

- (NSRect) flipNSCGExtendedDesktopBounds:(NSRect)bounds;
- (void) createHighlightWindow:(AXUIElementRef)overThisWindow;
- (void)activateStatusMenu;

- (void)setGrabbedUIWindow:(AXUIElementRef)uiWindow;
- (AXUIElementRef)grabbedUIWindow;
- (Boolean)isUIElementMovable:(AXUIElementRef)uiElement;
- (Boolean)isUIElementResizeable:(AXUIElementRef)uiElement;
- (Boolean)getUIElementBounds:(AXUIElementRef)uiElement origin:(CGPoint *)winOriginp size:(CGSize *)winSizep;
- (void)preserveCursorRelation:(CGPoint *)original_origin origSize:(CGSize *)original_size;
- (Boolean)moveUIElement:(AXUIElementRef)uiElement byAmount:(CGPoint *)pixeldistp;
- (Boolean)resizeUIElement:(AXUIElementRef)uiElement byAmount:(CGSize *)sizediffp;
- (Boolean)isaBannedWindow:(AXUIElementRef)uiElement;
- (int)isaStandardWindow:(AXUIElementRef)uiElement;
- (void)changeGrabbedWindowBounds:(NSRect *)by_amountp sourceHand:(FWHID_HandMotionPtr)onehandp;

- (AXUIElementRef)findStandardUIWindow:(AXUIElementRef)topElement;
- (void)grabFocusWindow;
- (void)grabWindowUnderPointer;
- (void)dropGrabbedWindow;

//highlightGrabbedWindow preference handling
- (void)toggleHighlightMenuItemState:(id)sender;
- (void)setHighlightGrabbedWindow:(BOOL)highlightOn;
- (BOOL)highlightGrabbedWindow;

//useKeyFocusWindow preference handling
- (void)toggleUseFocusWindow:(id)sender;
- (void)setUseKeyFocusWindow:(BOOL)use_focus_window;
- (BOOL)useKeyFocusWindow;

//orthoStretchAsDefault preference handling
- (void)setSizeAspectAsDefault:(id)sender;
- (void)setOrthoStretchAsDefault:(id)sender;
- (BOOL)orthoStretchAsDefault;

//dragByBothCorners preference handling
- (void)toggleBothCornersMenuItemState:(id)sender;
- (void)setDragByBothCorners:(BOOL)highlightOn;
- (BOOL)dragByBothCorners;

@end
