/*
 *  XWinderShell.m
 *  FingerWorks XWinder window manipulation utility (Mac OS X Implementation)
 *
 *  Created by Wayne Westerman on Sunday Nov 9 2003.
 *  Last Updated by Wayne Westerman on Friday Feb 27 2004.
 *  Copyright (c) 2003-2004 FingerWorks, Inc. All rights reserved.
 **/
/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is XWinderShell.m,
*   Mac OS X Implementation of XWinder window manipulation utility.
*
* The Initial Developer of the Original Code is
* Wayne Westerman, FingerWorks, Inc.
*
* Portions created by the Initial Developer are Copyright (C) 2003
* by FingerWorks Inc. All Rights Reserved.
*
* Contributor(s):
* Scott Menor
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

#import "XWinder_Prefix.pch"
#import "XWinderShell.h"

XWinderShell *gXWinderShell = NULL;

NSString *XWFWHighlightGrabbedWindowKey = @"HighlightGrabbedWindow";
NSString *XWFWUseKeyFocusWindowKey = @"UseKeyFocusWindow";
NSString *XWFWOrthoStretchAsDefaultKey = @"OrthoStretchAsDefault";
NSString *XWFWHoldBothCornersKey = @"HoldBothCorners";


static NSRect GetPrimaryDesktopBounds(NSRect bounds) {
    bounds = [[NSScreen mainScreen] frame];

    return bounds;
}

static NSRect GetExtendedDesktopBounds(NSRect bounds) {
	NSEnumerator *enumerator = [[NSScreen screens] objectEnumerator];
	NSScreen * scr = nil;
	float maxx = 0,maxy = 0;
	bounds.origin.x = 0;
	bounds.origin.y = 0;
	
	while (scr = (NSScreen *)[enumerator nextObject]) {
		if([scr frame].origin.x < bounds.origin.x)
			bounds.origin.x = [scr frame].origin.x;
		if([scr frame].origin.y < bounds.origin.y)
			bounds.origin.y = [scr frame].origin.y;
		if(([scr frame].origin.x + [scr frame].size.width) > maxx)
			maxx = ([scr frame].origin.x + [scr frame].size.width);
		if(([scr frame].origin.y + [scr frame].size.height) > maxy)
			maxy = ([scr frame].origin.y + [scr frame].size.height);
	}
	
	bounds.size.width = maxx - bounds.origin.x;
	bounds.size.height = maxy - bounds.origin.y;
	
	return bounds;
}

@implementation XWinderShell

- (NSRect) flipNSCGExtendedDesktopBounds:(NSRect)bounds {
	float screen_height = NSMaxY(_extendedDesktopBounds);
	bounds.origin.y = screen_height - NSMaxY(bounds);
    return bounds;
}

- (void) createHighlightWindow:(AXUIElementRef)overThisWindow {
    if (_highlightWindow != nil) {
        [_highlightWindow close];
        _highlightWindow = nil;
    }

    if (overThisWindow != NULL && [self highlightGrabbedWindow]) {
        //make the window the same position and size as overThisWindow
        id existingPosition = [InspectorWindow valueOfExistingAttribute:kAXPositionAttribute ofUIElement:overThisWindow];
        id existingSize = [InspectorWindow valueOfExistingAttribute:kAXSizeAttribute ofUIElement:overThisWindow];

        if (existingPosition && existingSize) {
            NSRect top_left_window_bounds;
            AXValueGetValue((CFTypeRef)existingSize, kAXValueCGSizeType, &top_left_window_bounds.size);
            AXValueGetValue((CFTypeRef)existingPosition, kAXValueCGPointType, &top_left_window_bounds.origin);
            _highlightWindow = [[NSWindow alloc] initWithContentRect:[self flipNSCGExtendedDesktopBounds:top_left_window_bounds] styleMask:NSBorderlessWindowMask backing:NSBackingStoreNonretained defer:YES];
            [_highlightWindow setOpaque:NO];
            [_highlightWindow setAlphaValue:0.25];
            [_highlightWindow setBackgroundColor:[NSColor greenColor]];
            [_highlightWindow setLevel:NSStatusWindowLevel];
            [_highlightWindow setReleasedWhenClosed:YES];
            [_highlightWindow orderFront:nil];
        }
    }
}

- (void)activateStatusMenu
{

    if(_statusBarMenu != nil) {
        [_statusBarMenu release];
        _statusBarMenu = nil;
    }
    _statusBarMenu = [[NSMenu alloc] init];
    NSMenuItem *item = [_statusBarMenu addItemWithTitle:@"Highlight Window" action:@selector(toggleHighlightMenuItemState:) keyEquivalent:@""];
    [item setTarget:self];
    if([self highlightGrabbedWindow])
        [item setState:NSOnState];
    else [item setState:NSOffState];

    NSMenuItem *itemBH = [_statusBarMenu addItemWithTitle:@"Both Hands Stretch Corners" action:@selector(toggleBothCornersMenuItemState:) keyEquivalent:@""];
    [itemBH setTarget:self];
    if([self dragByBothCorners])
        [itemBH setState:NSOnState];
    else [itemBH setState:NSOffState];

    NSMenuItem *itemFocus = [_statusBarMenu addItemWithTitle:@"Grab Key Focus Window" action:@selector(toggleUseFocusWindow:) keyEquivalent:@""];
    [itemFocus setTarget:self];
    if([self useKeyFocusWindow])
        [itemFocus setState:NSOnState];
    else [itemFocus setState:NSOffState];

    [_statusBarMenu addItem:[NSMenuItem separatorItem]];
    NSMenuItem *local_itemSizeAspect = [_statusBarMenu addItemWithTitle:@"Size/Aspect Unless 5 Fing" action:@selector(setSizeAspectAsDefault:) keyEquivalent:@""];
    _itemSizeAspect = local_itemSizeAspect;
    [_itemSizeAspect setTarget:self];
    if([self orthoStretchAsDefault])
        [_itemSizeAspect setState:NSOffState];
    else [_itemSizeAspect setState:NSOnState];

    NSMenuItem *local_itemOrthoStretch = [_statusBarMenu addItemWithTitle:@"Ortho Stretch Unless 5 Fing" action:@selector(setOrthoStretchAsDefault:) keyEquivalent:@""];
    _itemOrthoStretch = local_itemOrthoStretch;
    [_itemOrthoStretch setTarget:self];
    if([self orthoStretchAsDefault])
        [_itemOrthoStretch setState:NSOnState];
    else [_itemOrthoStretch setState:NSOffState];

    [_statusBarMenu addItem:[NSMenuItem separatorItem]];
    NSMenuItem *about_item = [_statusBarMenu addItemWithTitle:@"About XWinder" action:@selector(orderFrontStandardAboutPanel:) keyEquivalent:@""];
    [about_item setTarget:NSApp];
    NSMenuItem *quit_item = [_statusBarMenu addItemWithTitle:@"Quit XWinder" action:@selector(terminate:) keyEquivalent:@"q"];
    [quit_item setTarget:NSApp];

    NSStatusBar *bar = [NSStatusBar systemStatusBar];
    if(_statusBarItem != nil) {
        [bar removeStatusItem: _statusBarItem];
        [_statusBarItem release];
        _statusBarItem = nil;
    }

    _statusBarItem = [bar statusItemWithLength:NSSquareStatusItemLength];
    [_statusBarItem retain];

    [_statusBarItem setImage: [NSImage imageNamed: @"XWinderStatusIcon"]];
    [_statusBarItem setToolTip:@"FingerWorks XWinder"];
    [_statusBarItem setHighlightMode:YES];
    [_statusBarItem setMenu:_statusBarMenu];
}

- (void)awakeFromNib
{
    gXWinderShell = self;
			
	
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSDictionary *appDefaults = [NSDictionary dictionaryWithObject:@"NO" forKey:XWFWHighlightGrabbedWindowKey];
	[defaults registerDefaults:appDefaults];
	appDefaults = [NSDictionary dictionaryWithObject:@"NO" forKey:XWFWUseKeyFocusWindowKey];
	[defaults registerDefaults:appDefaults];
	appDefaults = [NSDictionary dictionaryWithObject:@"YES" forKey:XWFWOrthoStretchAsDefaultKey];
	[defaults registerDefaults:appDefaults];
	appDefaults = [NSDictionary dictionaryWithObject:@"YES" forKey:XWFWHoldBothCornersKey];
	[defaults registerDefaults:appDefaults];
		

    // Check if the Accessibility APIs are turned on.  If not, tell the user to do it (they'll need to authenticate to do it).
    if (!AXAPIEnabled())
    {
        int returncode = NSRunAlertPanel (@"XWinder will not work without the Accessibility API enabled.  Would you like to launch System Preferences?  Then you can turn on \"Enable access for assistive devices\".", @"", @"OK", @"Quit XWinder", @"Cancel");
        if(returncode == NSAlertAlternateReturn) {
            [NSApp terminate:self];
            return;
        } else if(returncode == NSAlertDefaultReturn) {
            [[NSWorkspace sharedWorkspace] openFile:@"/System/Library/PreferencePanes/UniversalAccessPref.prefPane"];
        } else if(returncode == NSAlertOtherReturn) {
            //continue without fixing preferences
        }
    }

    [_inspectorWindow setHidesOnDeactivate:NO];
    _primaryDesktopBounds = GetPrimaryDesktopBounds(_primaryDesktopBounds);
    _extendedDesktopBounds = GetExtendedDesktopBounds(_extendedDesktopBounds);
	
    _systemWideElement = AXUIElementCreateSystemWide();
    FWHID_registerMultiTouchDevices(NULL);
    FWHID_setHandEventCallback(XWFWhandleHandMotion);

    //[NSThread setThreadPriority:1.0];
    //printf("Priority %f\n",[NSThread threadPriority]);
    [self activateStatusMenu];
}


- (void)setGrabbedUIWindow:(AXUIElementRef)uiWindow
{
    [self createHighlightWindow:uiWindow];
    if (uiWindow)
        CFRetain( uiWindow );

    if (_grabbedUIWindowRef)
        CFRelease( _grabbedUIWindowRef );

    _grabbedUIWindowRef = uiWindow;
}

- (AXUIElementRef)grabbedUIWindow
{
    return _grabbedUIWindowRef;
}

- (Boolean)isUIElementResizeable:(AXUIElementRef)uiElement
{
    Boolean sizeSettable = false;
    if (uiElement) {
        AXUIElementIsAttributeSettable( uiElement, kAXSizeAttribute, &sizeSettable );
    }
    return sizeSettable;
}

- (Boolean)isUIElementMovable:(AXUIElementRef)uiElement
{
    Boolean positionSettable = false;
    if (uiElement) {
        AXUIElementIsAttributeSettable( uiElement, kAXPositionAttribute, &positionSettable );
    }
    return positionSettable;
}

- (Boolean)getUIElementBounds:(AXUIElementRef)uiElement origin:(CGPoint *)winOriginp size:(CGSize *)winSizep
{
    CFTypeRef	sizeValue 	= NULL;
    CFTypeRef	positionValue 	= NULL;
    Boolean query_successful = false;
    if(AXUIElementCopyAttributeValue( uiElement, kAXSizeAttribute, &sizeValue ) == kAXErrorSuccess
       && AXUIElementCopyAttributeValue( uiElement, kAXPositionAttribute, &positionValue ) == kAXErrorSuccess) {
        if (AXValueGetType(sizeValue) == kAXValueCGSizeType
            && AXValueGetType(positionValue) == kAXValueCGPointType
            && AXValueGetValue(sizeValue, kAXValueCGSizeType, winSizep)
            && AXValueGetValue(positionValue, kAXValueCGPointType, winOriginp)) {
            query_successful = true;
        }
    }
    return query_successful;
}


- (void)preserveCursorRelation:(CGPoint *)original_origin origSize:(CGSize *)original_size
{
	CGPoint new_origin;
	CGSize new_size;

	NSPoint existing_cursor_location = [NSEvent mouseLocation];
	
	existing_cursor_location.y = _primaryDesktopBounds.size.height - existing_cursor_location.y;

	if(original_size->width > 0 && original_size->height > 0
	   && [self getUIElementBounds:_grabbedUIWindowRef origin:&new_origin size:&new_size]
	   && new_size.width > 0 && new_size.height > 0) {
		//keep the cursor roughly the same relative distance to window edges
		//  as before the window bounds change
		float width_ratio = (float)(existing_cursor_location.x - original_origin->x)/original_size->width;
		float height_ratio = (float)(existing_cursor_location.y - original_origin->y)/original_size->height;
		CGPoint new_cursor_location;
		new_cursor_location.x = new_origin.x + width_ratio*new_size.width;
		new_cursor_location.y = new_origin.y + height_ratio*new_size.height;
		
		CGWarpMouseCursorPosition(new_cursor_location);
	}
}


- (Boolean)moveUIElement:(AXUIElementRef)uiElement byAmount:(CGPoint *)pixeldistp
{
    Boolean move_successful = false;

    if([self isUIElementMovable:uiElement]
       && (pixeldistp->x != 0 || pixeldistp->y != 0)) {
        CGPoint elorigin;
        CGSize elsizo;
        if([self getUIElementBounds:uiElement origin:&elorigin size:&elsizo]) {
            CFTypeRef	valueRef = NULL;
            CGPoint neworigin;
	
	    /* not using these but left them here in case someone wants to use them in the future; 
			note - they may not be obvious because of a change in the coordinate system 
				(in the _extendedDesktopBounds coordinate system, the origin is the bottom left of the primary display with y increasing as vertical position increases; 
				 in the neworigin coordinate system, the top left of the primary display is the origin with y increasing as vertical position decreases) */
            float right_extended_desktop_border = -25 + _extendedDesktopBounds.origin.x + _extendedDesktopBounds.size.width;
            float left_extended_desktop_border = 25 + _extendedDesktopBounds.origin.x;
            float bottom_extended_desktop_border = -20 + (_primaryDesktopBounds.size.height-_primaryDesktopBounds.origin.y) - (_extendedDesktopBounds.origin.y);
            float top_extended_desktop_border = 20 + (_primaryDesktopBounds.size.height-_primaryDesktopBounds.origin.y) - (_extendedDesktopBounds.size.height-_extendedDesktopBounds.origin.y);
	
			float right_primary_desktop_border = _primaryDesktopBounds.origin.x + _primaryDesktopBounds.size.width;
            float left_primary_desktop_border = _primaryDesktopBounds.origin.x;
			//float bottom_primary_desktop_border = -20 + _primaryDesktopBounds.size.height;
            float top_primary_desktop_border = 20.0; // TODO - verify that this is safe
	
            //CGPoint originalorigin = elorigin;
            neworigin.x = elorigin.x + pixeldistp->x;
            neworigin.y = elorigin.y + pixeldistp->y;
            if((neworigin.x > elorigin.x && neworigin.x > right_extended_desktop_border)) {
                //don't go off right screen edge
                if(elorigin.x > right_extended_desktop_border)
                    neworigin.x = elorigin.x; 
                else
                    neworigin.x = right_extended_desktop_border; 
            } else if(neworigin.x < elorigin.x && neworigin.x + elsizo.width < left_extended_desktop_border) {
                //or left screen edge
                if(elorigin.x + elsizo.width < left_extended_desktop_border)
                    neworigin.x = elorigin.x; 
                else
                    neworigin.x = left_extended_desktop_border - elsizo.width; 
            } else if(neworigin.y > elorigin.y && neworigin.y > bottom_extended_desktop_border) {
                //or bottom screen edge
                if(elorigin.y > bottom_extended_desktop_border) 
                    neworigin.y = elorigin.y;                
                else
                    neworigin.y = bottom_extended_desktop_border;
            } else if(neworigin.y < elorigin.y && neworigin.y < top_extended_desktop_border) {
                //or go past top menubar
                if(elorigin.y < top_extended_desktop_border) 
                    neworigin.y = elorigin.y;                
                else
                    neworigin.y = top_extended_desktop_border;
            }
			
			if(neworigin.x > left_primary_desktop_border && neworigin.x + elsizo.width < right_primary_desktop_border) {
				//window is clearly within primary desktop horizontal range, so constrain below top menu.  
				// WARNING: if you arrange a secondary display to be ABOVE the primary then the code
				//   below will prevent you from dragging a window from primary up to secondary !?
				//   In that case you can comment out these statements...
				if(neworigin.y < elorigin.y && neworigin.y < top_primary_desktop_border) {					
					if(elorigin.y < top_primary_desktop_border) 
						neworigin.y = elorigin.y;  //already up there but don't go higher              
					else //clip position to top menubar
						neworigin.y = top_primary_desktop_border;
				}				
			}
				
            valueRef = AXValueCreate( kAXValueCGPointType, (const void *)&neworigin );
            if (valueRef) {
                AXUIElementSetAttributeValue( uiElement, kAXPositionAttribute, valueRef );
                move_successful = true;
                /*printf("Moved window by [%2.0f, %2.0f], from [%3.0f, %3.0f] to [%3.0f, %3.0f]\n",
                    pixeldistp->x,pixeldistp->y,originalorigin.x, originalorigin.y,elorigin.x,
                    elorigin.y);                            */
                CFRelease( valueRef );
            }
        }
    }
    return move_successful;
}

- (Boolean)resizeUIElement:(AXUIElementRef)uiElement byAmount:(CGSize *)sizediffp
{
    Boolean resize_successful = false;
    Boolean size_can_change = false;
    if([self isUIElementResizeable:uiElement]
       && (sizediffp->width != 0 || sizediffp->height != 0)) {
        CGPoint elorigin;
        CGSize elsizo;

        if([self getUIElementBounds:uiElement origin:&elorigin size:&elsizo]) {
            //CGSize originalsize = elsizo;
            if(((elsizo.width + sizediffp->width) > 200 || sizediffp->width > 0)    //don't reduce width to less than 200
               && ((elsizo.width + sizediffp->width + elorigin.x) > (_extendedDesktopBounds.origin.x + 30) //make sure right window edge doesn't go off left screen edge
                   || sizediffp->width < 0)) {
                elsizo.width += sizediffp->width;
                size_can_change = true;
            }
            if((elsizo.height + sizediffp->height > 150 || sizediffp->height > 0) //don't reduce height to less than 150
               &&  (elsizo.height + sizediffp->height<= _extendedDesktopBounds.size.height
                    || sizediffp->height < 0)) {
                elsizo.height += sizediffp->height;
                size_can_change = true;
            }
            if(size_can_change) {
                CFTypeRef	valueRef = NULL;
                valueRef = AXValueCreate( kAXValueCGSizeType, (const void *)&elsizo );
                if (valueRef) {
                    AXUIElementSetAttributeValue( uiElement, kAXSizeAttribute, valueRef );
                    resize_successful = true;
                    /*printf("\tResized window by [%2.0f, %2.0f], from [%3.0f, %3.0f] to [%3.0f, %3.0f]\n",
                           sizediffp->width,sizediffp->height,originalsize.width,originalsize.height,elsizo.width,
                           elsizo.height);                            */
                    CFRelease( valueRef );
                }
            }
        }
    }
    return resize_successful;
}


- (void)changeGrabbedWindowBounds:(NSRect*)by_amountp sourceHand:(FWHID_HandMotionPtr)onehandp {
    if(_grabbedUIWindowRef) {
        CGPoint original_origin;
        CGSize original_size;
		Boolean wanna_warp_cursor = false;

        if([self getUIElementBounds:_grabbedUIWindowRef origin:&original_origin size:&original_size])
			wanna_warp_cursor = true;
			/* If the API supported it, ideally we'ld want to use a call that moved and resized the window at the same time,
            i.e. a setFrameBounds() call.  But Accessibility API doesn't let us do that :( */

        //we get much less screen flicker if we resize (the slower operation) before move (instantaneous!)
        //printf("\tTrying to resize by [%2.0f, %2.0f]\n",resize_amount.width,resize_amount.height);
        if([self resizeUIElement:_grabbedUIWindowRef byAmount:(CGSize*)&by_amountp->size]) {
            //printf("\tResized window by w=%f, h=%f\n",resize_amount.width,resize_amount.height);
        }
        //printf("Trying to move by [%2.0f, %2.0f]\n",move_amount.x,move_amount.y);
        if([self moveUIElement:_grabbedUIWindowRef byAmount:(CGPoint*)&by_amountp->origin]) {
            //printf("Moved by [%2.0f, %2.0f]\n",move_amount.x,move_amount.y);
        }
        if(wanna_warp_cursor && ![self useKeyFocusWindow]) {
			//printf("Moved by [%2.0f, %2.0f]\n",move_amount.x,move_amount.y);
			[self preserveCursorRelation:&original_origin origSize:&original_size];
		}

        if([self highlightGrabbedWindow]) {
            if(by_amountp->size.width != 0 || by_amountp->size.height != 0) {
                [self createHighlightWindow:_grabbedUIWindowRef];
            } else {
                NSRect updatedWindowRect;
                if(_highlightWindow
                   && [self getUIElementBounds:_grabbedUIWindowRef origin:(CGPoint*)&updatedWindowRect.origin size:(CGSize*)&updatedWindowRect.size]) {
                    [_highlightWindow orderFrontRegardless];
                    [_highlightWindow setFrame:[self flipNSCGExtendedDesktopBounds:updatedWindowRect] display:false];
                    [_highlightWindow orderFrontRegardless];
                    //[_highlightWindow setFrame:[_grabbedUIWindow frame] display:false];
                }
            }
        }
        //fflush(stdout);
    }
}

- (Boolean)isaBannedWindow:(AXUIElementRef)uiWindow
{
    Boolean is_banned_window = false;
    if(uiWindow) {
        AXUIElementRef elparento = (AXUIElementRef)[InspectorWindow valueOfExistingAttribute:kAXParentAttribute ofUIElement:uiWindow];
        NSString *parentRole = NULL;
        if (AXUIElementCopyAttributeValue( (AXUIElementRef)elparento, kAXRoleAttribute, (CFTypeRef *)&parentRole ) == kAXErrorSuccess) {
            if([parentRole isEqualToString:(NSString *)kAXApplicationRole]) {
                NSString *parentTitle = [InspectorWindow valueOfExistingAttribute:kAXTitleAttribute ofUIElement:(AXUIElementRef)elparento];
                if([parentTitle isEqualToString:(NSString *)@"Word"]) {
                    is_banned_window = true;  //found Microsoft Word window, which does not redraw properly if we try to move it
                    printf("Found Microsoft Word window!  Ignoring\n");
                } else if([parentTitle isEqualToString:(NSString *)@"Finder"]
                            && ![self isUIElementResizeable:uiWindow]) {
                    is_banned_window = true; //found Finger desktop background window, which we probably shouldn't try to resize :)
					printf("Found Finder desktop window!  Ignoring\n");
                }
            }
            [parentRole release];
        }
    }
    return is_banned_window;
}


- (int)isaStandardWindow:(AXUIElementRef)uiElement
{
    int isDirectableWindow = 0;
    if (uiElement) {
        NSString *uiElementRole = NULL;

        if (AXUIElementCopyAttributeValue( (AXUIElementRef)uiElement, kAXRoleAttribute, (CFTypeRef *)&uiElementRole ) == kAXErrorSuccess) {
            if([uiElementRole isEqualToString:(NSString *)kAXWindowRole]) {

                //now must check parent element for incompatible applications
                if([self isaBannedWindow:uiElement])
                    isDirectableWindow = -1;
                else
                    isDirectableWindow = 1;

            } else if([uiElementRole isEqualToString:(NSString *)kAXApplicationRole]) {
                isDirectableWindow = -1;
            }
            [uiElementRole release];
        }
    }
    return isDirectableWindow;
}

- (AXUIElementRef)findStandardUIWindow:(AXUIElementRef)topElement
{
    int already_iswindow = [self isaStandardWindow:topElement];

    if(already_iswindow == 1) {
		//printf("... got it ...\n");
        //we are already it!
    } else if(already_iswindow == 0) {
        //child is not a window UIElement, so recurse up the parent window attribute
        AXUIElementRef elparento = (AXUIElementRef)[InspectorWindow valueOfExistingAttribute:kAXParentAttribute ofUIElement:topElement];
        if(elparento != NULL) {
            //printf("... recursing ...\n");
            topElement = [self findStandardUIWindow:elparento];
		}
    } else if(already_iswindow == -1) {
        //hit Application role, so give up!
        //printf("Window search giving up!");
        topElement = nil;
    } else {
		//printf("find what?!!\n");				
	}
    return topElement;
}

- (void)grabFocusWindow {
    //printf("grabFocusWindow is not implemented yet!  Wanna help?\n");
    AXUIElementRef appwithfocus = (AXUIElementRef)[InspectorWindow valueOfExistingAttribute:kAXFocusedApplicationAttribute ofUIElement:_systemWideElement];
    if(appwithfocus != NULL) {
        AXUIElementRef windowwithfocus = (AXUIElementRef)[InspectorWindow valueOfExistingAttribute:kAXFocusedWindowAttribute ofUIElement:appwithfocus];
        if(windowwithfocus)
            windowwithfocus = [self findStandardUIWindow:windowwithfocus];
        if(windowwithfocus) {
            [self setGrabbedUIWindow:windowwithfocus];
            [_inspectorWindow displayInfoForUIElement:windowwithfocus];
            return; //success!
        }
    }
    //couldn't find grabbable window with key focus, so try window under pointer
    [_inspectorWindow displayInfoForUIElement:_systemWideElement];
    [self grabWindowUnderPointer];
}

- (void)grabWindowUnderPointer {
    Point		pointAsCarbonPoint;
    
	//printf("trying grabWindowUnderPointer()...\n");
	
    // The current mouse position with origin at top left.
    GetMouse( &pointAsCarbonPoint );
	LocalToGlobal(&pointAsCarbonPoint);
    CGPoint				pointAsCGPoint;
    AXUIElementRef 		newElement		= NULL;

    pointAsCGPoint.x = pointAsCarbonPoint.h;
    pointAsCGPoint.y = pointAsCarbonPoint.v;

    // Ask Accessibility API for UI Element under the mouse
    // And update the display if a different UIElement
    if (AXUIElementCopyElementAtPosition( _systemWideElement, pointAsCGPoint.x, pointAsCGPoint.y, &newElement ) == kAXErrorSuccess) {
        if(newElement) {
            newElement = [self findStandardUIWindow:newElement];
		}
        if(newElement) {
            [self setGrabbedUIWindow:newElement];
            [_inspectorWindow displayInfoForUIElement:newElement];
        }
    } else {
		//printf("XWinder:  AXUIElementCopyElementAtPosition() failed!\n");
	}

}

- (void)dropGrabbedWindow {
    [self setGrabbedUIWindow:nil];
}

- (void)toggleHighlightMenuItemState:(id)sender {
    if([sender state] == NSOnState) {
        [sender setState:NSOffState];
        [sender setTitle:@"Highlight: Off"];
        [self setHighlightGrabbedWindow:false];
    } else {
        [sender setState:NSOnState];
        [sender setTitle:@"Highlight: On"];
        [self setHighlightGrabbedWindow:true];
    }
}

//highlightGrabbedWindow preference handling

- (void)setHighlightGrabbedWindow:(BOOL)highlightOn {
	[[NSUserDefaults standardUserDefaults] setBool:highlightOn forKey:XWFWHighlightGrabbedWindowKey];
}

- (BOOL)highlightGrabbedWindow {
    return [[NSUserDefaults standardUserDefaults] boolForKey:XWFWHighlightGrabbedWindowKey];
}


//useKeyFocusWindow preference handling

- (void)toggleUseFocusWindow:(id)sender {
    if([sender state] == NSOnState) {
        [sender setState:NSOffState];
		[self setUseKeyFocusWindow:false];
    } else {
        [sender setState:NSOnState];
		[self setUseKeyFocusWindow:true];
    }
}

- (void)setUseKeyFocusWindow:(BOOL)use_focus_window {
	[[NSUserDefaults standardUserDefaults] setBool:use_focus_window forKey:XWFWUseKeyFocusWindowKey];
}

- (BOOL)useKeyFocusWindow {
    return [[NSUserDefaults standardUserDefaults] boolForKey:XWFWUseKeyFocusWindowKey];
}

//orthoStretchAsDefault preference handling

- (void)setOrthoStretchAsDefault:(id)sender {
	[[NSUserDefaults standardUserDefaults] setBool:true forKey:XWFWOrthoStretchAsDefaultKey];
    if(_itemSizeAspect != nil) {
        [_itemSizeAspect setState:NSOffState];
    }
    if(_itemOrthoStretch != nil) {
        [_itemOrthoStretch setState:NSOnState];
    }
}

- (void)setSizeAspectAsDefault:(id)sender {
	[[NSUserDefaults standardUserDefaults] setBool:false forKey:XWFWOrthoStretchAsDefaultKey];
    if(_itemSizeAspect != nil) {
        [_itemSizeAspect setState:NSOnState];
    }
    if(_itemOrthoStretch != nil) {
        [_itemOrthoStretch setState:NSOffState];
    }
}

- (BOOL)orthoStretchAsDefault {
    return [[NSUserDefaults standardUserDefaults] boolForKey:XWFWOrthoStretchAsDefaultKey];
}


//dragByBothCorners preference handling

- (void)toggleBothCornersMenuItemState:(id)sender {
    if([sender state] == NSOnState) {
        [sender setState:NSOffState];
        [self setDragByBothCorners:false];
    } else {
        [sender setState:NSOnState];
        [self setDragByBothCorners:true];
    }
}

- (void)setDragByBothCorners:(BOOL)bothCornersOn {
	[[NSUserDefaults standardUserDefaults] setBool:bothCornersOn forKey:XWFWHoldBothCornersKey];
}

- (BOOL)dragByBothCorners {
	return [[NSUserDefaults standardUserDefaults] boolForKey:XWFWHoldBothCornersKey];
}


@end

//C Wrappers to Objective C methods
void XWFWgrabWindow(FWHID_HandMotionPtr movinghandp) {
    if([gXWinderShell useKeyFocusWindow])
        [gXWinderShell grabFocusWindow];
    else
        [gXWinderShell grabWindowUnderPointer];
}

void XWFWdropGrabbedWindow(FWHID_HandMotionPtr movinghandp) {
    [gXWinderShell dropGrabbedWindow];
}

void XWFWchangeGrabbedWindowBounds(XWFWRect *by_amountp, FWHID_HandMotionPtr movinghandp) {
    [gXWinderShell changeGrabbedWindowBounds:(NSRect *)by_amountp sourceHand:movinghandp];
}

void XWFWpanWindow(int byamount) {
	//ignore pan calls on Mac OS X since the OS mouse driver handles it OK (Unlike WinXP!)
	//do nothing...
}

void XWFWhandleActionCode(int action_code) {
	//Mac OS X Accessibility API doesn't give enough window control to do maximize/restore/toggle/send-to-back, etc.
	//do nothing for now...
}


/**--- Mac OS X preference query implementations  **/
bool XWFWgetBooleanPreference(char *preference_name) {
    if(strcmp(preference_name,"HoldBothCorners") == 0) {
       return [gXWinderShell dragByBothCorners];
    } else if(strcmp(preference_name,"OrthoStretchAsDefault") == 0) {
       return [gXWinderShell orthoStretchAsDefault];
    } else if(strcmp(preference_name,"UseKeyFocusWindow") == 0) {
		return [gXWinderShell useKeyFocusWindow];
    } else if(strcmp(preference_name,"HighlightGrabbedWindow") == 0) {
		return [gXWinderShell highlightGrabbedWindow];
    } else return false;
}

XWFWTimeInterval max_windRefreshInterval_seconds = .025;

double XWFWgetDoublePreference(char *preference_name) {
    if(strcmp(preference_name,"BimanualRefreshDelay") == 0) {
       return max_windRefreshInterval_seconds;
	} else if(strcmp(preference_name,"UnimanualRefreshDelay") == 0) {
		return 0;  //wind timers create lots of sluggishness if used with one-handed winding?
    } else return 0;
}

/*
void XWFWsetIntegerPreference(char *preference_name, int value) {
    if(strcmp(preference_name,"MagneticEdgeStrength") == 0) {
		gMagneticEdgeStrength = value;
		xwfw_setRegistryDword(preference_name,value);
	}
}

int XWFWgetIntegerPreference(char *preference_name) {
    if(strcmp(preference_name,"MagneticEdgeStrength") == 0) {
       return gMagneticEdgeStrength;                
    } else return 0;
}*/


/**--- Mac OS X preference timer/delay implementations  **/

void xwfw_postponedWindCallback(CFRunLoopTimerRef timer, void *context);

XWFWAbsoluteTime XWFWgetAbsoluteTime(void) {
	return CFAbsoluteTimeGetCurrent();
}

static CFRunLoopTimerRef gXWFWpostponedWindTimer = NULL;
static XWFWMotionOnlyCALLBACK gXWFWwindNowCallback = NULL;

void XWFWstopPostponedWindTimer(void) {
    if(gXWFWpostponedWindTimer != NULL) {
        CFRunLoopTimerInvalidate(gXWFWpostponedWindTimer);
        CFRelease(gXWFWpostponedWindTimer);
        gXWFWpostponedWindTimer = NULL;
    }
}

void xwfw_postponedWindCallback(CFRunLoopTimerRef timer, void *context) {
    CFRunLoopTimerContext *timercontextp = context;
    FWHID_HandMotionPtr pendingSourceHandp = NULL;
    if(timercontextp != NULL && timercontextp->info != NULL)
        pendingSourceHandp = timercontextp->info;

    (*gXWFWwindNowCallback)(pendingSourceHandp);
    XWFWstopPostponedWindTimer();
}

void XWFWstartPostponedWindTimer(XWFWMotionOnlyCALLBACK windNowCallback,
	FWHID_HandMotionPtr pendingSourceHandp, XWFWTimeInterval delay) {
    /* Only resteart a new timer if one not going yet.. */
    if(gXWFWpostponedWindTimer == NULL) {
        CFRunLoopTimerContext timerCtxt;
        gXWFWwindNowCallback = windNowCallback;
        memset(&timerCtxt, 0, sizeof(CFRunLoopTimerContext));
        timerCtxt.info = pendingSourceHandp;
        gXWFWpostponedWindTimer = CFRunLoopTimerCreate(NULL,
                                                   CFAbsoluteTimeGetCurrent() + delay,
                                                   0, // interval 0 doesn't repeat
                                                   0,   // flags
                                                   0,   // order index
                                                   (CFRunLoopTimerCallBack)xwfw_postponedWindCallback,
                                                   &timerCtxt);
        if(gXWFWpostponedWindTimer) {
            CFRunLoopAddTimer(CFRunLoopGetCurrent(), gXWFWpostponedWindTimer, kCFRunLoopCommonModes);
        }
    }
}