/*
 *  XWinderShell.c
 *  FingerWorks XWinder window manipulation utility (Windows Implementation)
 *
 *  Created by Wayne Westerman on Sunday Nov 9 2003.
 *  Last Updated by Wayne Westerman on Wednesday Feb 11 2004.
 *  Copyright (c) 2003-2004 FingerWorks, Inc. All rights reserved. 
 **/
/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is XWinderShell.c, 
*   Windows Implementation of XWinder window manipulation utility.
*
* The Initial Developer of the Original Code is
* Wayne Westerman, FingerWorks, Inc.
*
* Portions created by the Initial Developer are Copyright (C) 2003-2004
* by FingerWorks Inc. All Rights Reserved.
*
* Contributor(s):
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

/* #ifndef _WIN32_IE    
#define _WIN32_IE 0x0600
 #endif
 #ifndef _WIN32_WINNT    
// Indicate this as a Windows XP app to pull in RawInput from Windows.h
#define _WIN32_WINNT 0x0501
 #endif
 */

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "FWHID_HandMotion.h"
#include "XWinderCalcs.h"

#include "resource.h"

#include "LimitSingleInstance.h"

// The one and only CLimitSingleInstance object.
CLimitSingleInstance g_SingleInstanceObj(TEXT("XWinder_1_0"));

#ifdef __cplusplus
extern "C" {
#endif

#define WM_XWINDERSHELLICON (WM_APP + 25)

static HWND gGrabbedWindow = NULL;
static HWND gHwndMain = NULL; //global handle for the main msg window
bool gStretchByBothCorners = true;
bool gUseKeyFocusWindow = false;
bool gOrthoStretchAsDefault = true;
XWFWTimeInterval max_windRefreshInterval = .040; //80 milliseconds on Mac OS X?

/**--- Win32 preference query implementations  **/
LONG xwfw_getRegistryDword(char *preference_name, DWORD *dword_valuep) {
	LONG return_val;
	DWORD dword_length = 4;
	DWORD value_type;
	HKEY hkey_software;
	if((return_val=RegOpenKeyEx(HKEY_CURRENT_USER,"Software",0,KEY_READ,&hkey_software)) == ERROR_SUCCESS) {
		HKEY hkey_fingerworks;
		if((return_val=RegOpenKeyEx(hkey_software,"FingerWorks",0,KEY_READ,&hkey_fingerworks)) == ERROR_SUCCESS) {
			HKEY hkey_XWinder;
			if((return_val=RegOpenKeyEx(hkey_fingerworks,"XWinder",0,KEY_READ,&hkey_XWinder)) == ERROR_SUCCESS) {
				return_val = RegQueryValueEx(hkey_XWinder,preference_name,0,&value_type,(LPBYTE)dword_valuep,&dword_length);
				RegCloseKey(hkey_XWinder);
			}
			RegCloseKey(hkey_fingerworks);
		}
		RegCloseKey(hkey_software);
	}
	return return_val;
}

LONG xwfw_setRegistryDword(char *preference_name, DWORD dword_value) {
	LONG return_val;
	HKEY hkey_software;
	if((return_val=RegOpenKeyEx(HKEY_CURRENT_USER,"Software",0,KEY_CREATE_SUB_KEY,&hkey_software)) == ERROR_SUCCESS) {
		HKEY hkey_fingerworks;
		if((return_val=RegCreateKeyEx(hkey_software,"FingerWorks",0,NULL,REG_OPTION_NON_VOLATILE,KEY_CREATE_SUB_KEY,NULL,&hkey_fingerworks,NULL)) == ERROR_SUCCESS) {
			HKEY hkey_XWinder;
			if((return_val=RegCreateKeyEx(hkey_fingerworks,"XWinder",0,NULL,REG_OPTION_NON_VOLATILE,KEY_CREATE_SUB_KEY | KEY_SET_VALUE,NULL,&hkey_XWinder,NULL)) == ERROR_SUCCESS) {
				return_val=RegSetValueEx(hkey_XWinder,preference_name,0,REG_DWORD,(BYTE*)&dword_value,4);
				RegCloseKey(hkey_XWinder);
			}
			RegCloseKey(hkey_fingerworks);
		}
		RegCloseKey(hkey_software);
	}
	return return_val;
}


void xwfw_initializePreferences(void) {
	DWORD value;
	if(xwfw_getRegistryDword("HoldBothCorners",&value) == ERROR_SUCCESS) {
		gStretchByBothCorners = (bool)value;
	}
	if(xwfw_getRegistryDword("OrthoStretchAsDefault",&value) == ERROR_SUCCESS) {
		gOrthoStretchAsDefault = (bool)value;
	}
	if(xwfw_getRegistryDword("UseKeyFocusWindow",&value) == ERROR_SUCCESS) {
		gUseKeyFocusWindow = (bool)value;
	}
}


bool XWFWgetBooleanPreference(char *preference_name) {
    if(strcmp(preference_name,"HoldBothCorners") == 0) {
       return gStretchByBothCorners;
	} else if(strcmp(preference_name,"OrthoStretchAsDefault") == 0) {
	   return gOrthoStretchAsDefault;
	} else if(strcmp(preference_name,"UseKeyFocusWindow") == 0) {
	   return gUseKeyFocusWindow;
    } else return false;
}

void XWFWsetBooleanPreference(char *preference_name, bool value) {
    if(strcmp(preference_name,"HoldBothCorners") == 0) {
		gStretchByBothCorners = value;
		xwfw_setRegistryDword(preference_name,value);
	} else if(strcmp(preference_name,"OrthoStretchAsDefault") == 0) {
		gOrthoStretchAsDefault = value;
		xwfw_setRegistryDword(preference_name,value);
	} else if(strcmp(preference_name,"UseKeyFocusWindow") == 0) {
		gUseKeyFocusWindow = value;
		xwfw_setRegistryDword(preference_name,value);
	}
}

double XWFWgetDoublePreference(char *preference_name) {
    if(strcmp(preference_name,"BimanualRefreshDelay") == 0) {
       return max_windRefreshInterval;                
	} else if(strcmp(preference_name,"UnimanualRefreshDelay") == 0) {
       return max_windRefreshInterval;                
    } else return 0;
}

/**--- Win32 timer/delay implementations  **/
XWFWAbsoluteTime XWFWgetAbsoluteTime(void) {
	return .001*GetTickCount();
}

static UINT_PTR gXWFWpostponedWindTimer = 0;
static XWFWMotionOnlyCALLBACK gXWFWwindNowCallback = NULL;
static FWHID_HandMotionPtr gXWFWpostponedSourceHandp = NULL;

static VOID CALLBACK xwfw_postponedWindCallback(HWND hwnd,
    UINT uMsg,
    UINT_PTR idEvent,
    DWORD dwTime) {    
    (*gXWFWwindNowCallback)(gXWFWpostponedSourceHandp);
    XWFWstopPostponedWindTimer();
}

void XWFWstopPostponedWindTimer(void) {
	KillTimer(gHwndMain, gXWFWpostponedWindTimer); 
	gXWFWpostponedWindTimer = 0;
}


void XWFWstartPostponedWindTimer(XWFWMotionOnlyCALLBACK windNowCallback,
										FWHID_HandMotionPtr pendingSourceHandp, XWFWTimeInterval delay)
{
	//this will reset 

    /* Only resteart a new timer if one not going yet.. */
    if(gXWFWpostponedWindTimer == 0) {
		gXWFWwindNowCallback = windNowCallback;
		gXWFWpostponedSourceHandp = pendingSourceHandp;
		gXWFWpostponedWindTimer = 77;      //arbitrary identifier 77
		SetTimer(gHwndMain,                // handle to main window 
				gXWFWpostponedWindTimer,               // timer identifier 
				(int)(1000.0*delay),                     //millisecond interval
		(TIMERPROC) xwfw_postponedWindCallback); // timer callback
    }
}


/**--- Win32 window grab/move/resize implementations  **/

void xwfw_sendToBack(HWND awindow) {
	SetWindowPos(awindow,HWND_BOTTOM,
				 0,0,0,0,
				 SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE | SWP_ASYNCWINDOWPOS);
}

void xwfw_bringToFront(HWND awindow) {
	//nothing works right!!!  Need SetActiveWindow()?
	/*OutputDebugString (TEXT("BringWindowToTop!\n")); 					
//	ShowWindow(awindow,SW_SHOW);
//	BringWindowToTop(awindow);
	SetWindowPos(awindow,
				 HWND_TOPMOST,0,0,0,0,
				 SWP_NOMOVE | SWP_NOSIZE | SWP_ASYNCWINDOWPOS);
	SetForegroundWindow(awindow);*/
}

void xwfw_setWindowState(HWND awindow, int handActionCode) {
	if(awindow != NULL) {
		int showCmd = SW_SHOWNORMAL;
		if(handActionCode == kFWHID_ActionCode_WinToggle) {
			if(IsZoomed(awindow)) {
				showCmd = SW_RESTORE;
				OutputDebugString (TEXT("Toggling to Restore!\n")); 					
			} else {
				showCmd = SW_SHOWMAXIMIZED;
				OutputDebugString (TEXT("Toggling to Maximize!\n")); 					
			}
		} else if(handActionCode == kFWHID_ActionCode_WinMaximize) {
			OutputDebugString (TEXT("Maximizing!\n")); 					
			showCmd = SW_SHOWMAXIMIZED;
		} else if(handActionCode == kFWHID_ActionCode_WinHide) {
			OutputDebugString (TEXT("Hiding!\n")); 					
			showCmd = SW_HIDE;
		} else if(handActionCode == kFWHID_ActionCode_WinMinimize) {
			OutputDebugString (TEXT("Minimizing!\n")); 					
			showCmd = SW_MINIMIZE;
		} else if(handActionCode == kFWHID_ActionCode_WinRestore) {
			OutputDebugString (TEXT("Restoring!\n")); 					
			showCmd = SW_RESTORE;
		}
		ShowWindow(awindow,showCmd);
	}
}


void XWFWhandleActionCode(int action_code) {
	if(gGrabbedWindow == NULL) 
		return;

	switch(action_code) {
		case kFWHID_ActionCode_WinBringToTop:
			xwfw_bringToFront(gGrabbedWindow);
			break;
		case kFWHID_ActionCode_WinSendToBottom:
			xwfw_sendToBack(gGrabbedWindow);
			break;
		case kFWHID_ActionCode_WinHide:
		case kFWHID_ActionCode_WinToggle:
		case kFWHID_ActionCode_WinMaximize:
		case kFWHID_ActionCode_WinMinimize:
		case kFWHID_ActionCode_WinRestore:
			xwfw_setWindowState(gGrabbedWindow,action_code);
			break;

		case kFWHID_ActionCode_SubWinSendToTop:
		case kFWHID_ActionCode_SubWinSendToBottom:
		case kFWHID_ActionCode_SubWinToggle:
		case kFWHID_ActionCode_SubWinMaximize:
		case kFWHID_ActionCode_SubWinMinimize:
		case kFWHID_ActionCode_SubWinRestore:
		default:
			break;
	}
}

//need to deMaximize windows before moving/resizing or they will think they're still maximized!!!
void xwfw_deMaximize(HWND awindow) {
	if(awindow != NULL && IsZoomed(awindow)) {
		WINDOWPLACEMENT wp;
		OutputDebugString (TEXT("De-maximizing before move!\n")); 					
		wp.length = sizeof(WINDOWPLACEMENT);
		if(GetWindowPlacement(awindow,&wp)) {
			//didn't fail
			RECT screenbounds;
			//window rect is in screen coordinates, and window placement in workspace coordinates
			// but we need window rect to compute maximized width/height!
			GetWindowRect(awindow,&screenbounds);
			//set normal bounds to maximized bounds so that when we 'Restore' to get out of 'Maximized'
			//  state the window won't go anywhere!
			wp.rcNormalPosition.left = wp.ptMaxPosition.x;
			wp.rcNormalPosition.top = wp.ptMaxPosition.y;
			wp.rcNormalPosition.right = wp.ptMaxPosition.x + (screenbounds.right - screenbounds.left);
			wp.rcNormalPosition.bottom = wp.ptMaxPosition.y + (screenbounds.bottom - screenbounds.top);
			wp.showCmd = SW_RESTORE;
			SetWindowPlacement(awindow,&wp);
		}
	}
}

BOOL xwfw_setGrabbedWindow(HWND candidateWindow) {
	if(candidateWindow != NULL) {
		//we may have grabbed a child/sub-window.  Find the root!
		HWND root_window = GetAncestor(candidateWindow,GA_ROOT);
		if(root_window != NULL) {
			candidateWindow = root_window;
			//OutputDebugString (TEXT("   Grabbed Root Ancestor!\n")); 
		}
	}

	if(candidateWindow != NULL) {
		//make sure we haven't grabbed desktop or taskbar, which should be fixed!
		TCHAR classnamebuf[100];
		RealGetWindowClass(candidateWindow,classnamebuf,100);
		//OutputDebugString (classnamebuf);
		if(lstrcmp(classnamebuf,"Progman") == 0) {
			OutputDebugString (TEXT("   Giving up Progman!\n")); 
			candidateWindow = NULL;
		} else if(lstrcmp(classnamebuf,"Shell_TrayWnd") == 0) {
			OutputDebugString (TEXT("   Giving up Shell_TrayWnd!\n")); 
			candidateWindow = NULL;
		} else if(candidateWindow == GetDesktopWindow()) {
			OutputDebugString (TEXT("   Giving up Desktop!\n")); 
			candidateWindow = NULL;
		}
	} 
	gGrabbedWindow = candidateWindow;
	return gGrabbedWindow != NULL;
}

void XWFWgrabWindow(FWHID_HandMotionPtr movinghandp) {
    if(gUseKeyFocusWindow)
        xwfw_setGrabbedWindow(GetForegroundWindow());
	if(!gUseKeyFocusWindow || gGrabbedWindow == NULL) {
		// we're in window under pointer mode or foreground window was not grabbable!
		POINT current_cursor_pos;
		if(GetCursorPos(&current_cursor_pos)) {
			xwfw_setGrabbedWindow(WindowFromPoint(current_cursor_pos));
		}
	}
}

void XWFWdropGrabbedWindow(FWHID_HandMotionPtr movinghandp) {
	gGrabbedWindow = NULL;
}

void xwfw_preserveCursorRelation(RECT* winbounds, RECT* oldbounds) {
	POINT old_cursor_pos;
	if(!gUseKeyFocusWindow && GetCursorPos(&old_cursor_pos)) {
		float old_width = (float)(oldbounds->right - oldbounds->left);
		float old_height = (float)(oldbounds->bottom - oldbounds->top);
		float new_width = (float)(winbounds->right - winbounds->left);
		float new_height = (float)(winbounds->bottom - winbounds->top);
		if(old_width > 0 && old_height > 0 && new_width > 0 && new_height > 0) {
			//keep the cursor roughly the same relative distance to window edges
			//  as before the window bounds change
			float width_ratio = (float)(old_cursor_pos.x - oldbounds->left)/old_width;
			float height_ratio = (float)(old_cursor_pos.y - oldbounds->top)/old_height;
			int newx = winbounds->left + (int)(width_ratio*new_width);
			int newy = winbounds->top + (int)(height_ratio*new_height);
			SetCursorPos(newx,newy);
		}
	}
}

void xwfw_limitWindowBounds(RECT* winbounds, RECT* oldbounds) {
	int virtualscreen_originx = GetSystemMetrics(SM_XVIRTUALSCREEN);
	int virtualscreen_originy = GetSystemMetrics(SM_YVIRTUALSCREEN);
	int virtualscreen_width = GetSystemMetrics(SM_CXVIRTUALSCREEN);
	int virtualscreen_height = GetSystemMetrics(SM_CXVIRTUALSCREEN);
	int wintrack_minwidth = GetSystemMetrics(SM_CXMINTRACK);
	int wintrack_minheight = GetSystemMetrics(SM_CYMINTRACK);
	int wintrack_maxwidth = GetSystemMetrics(SM_CXMAXTRACK);
	int wintrack_maxheight = GetSystemMetrics(SM_CYMAXTRACK);

	//check width so we don't get too big or small
	if((winbounds->right - winbounds->left) > wintrack_maxwidth
		|| (winbounds->right - winbounds->left) < wintrack_minwidth) {
		//retain old width while allowing move
		winbounds->right = winbounds->left + (oldbounds->right - oldbounds->left);
	}
	//check height so we don't get too big or small
	if((winbounds->bottom - winbounds->top) > wintrack_maxheight
		|| (winbounds->bottom - winbounds->top) < wintrack_minheight) {
		//retain old height while allowing move
		winbounds->bottom = winbounds->top + (oldbounds->bottom - oldbounds->top);
	}

	//check bounds to keep at least one window edge on-screen
	if(winbounds->right < virtualscreen_originx + 30) {
		int new_width = winbounds->right - winbounds->left;
		winbounds->right = oldbounds->right;
		//keep width as requested
		winbounds->left = winbounds->right - new_width;
	}
	if(winbounds->left > (virtualscreen_originx + virtualscreen_width - 30)) {
		int new_width = winbounds->right - winbounds->left;
		winbounds->left = oldbounds->left;
		//keep width as requested
		winbounds->right = new_width + winbounds->left;
	}
	if(winbounds->bottom < (virtualscreen_originy + 30)) {
		int new_height = winbounds->bottom - winbounds->top;
		winbounds->bottom = oldbounds->bottom;
		//keep height as requested
		winbounds->top = winbounds->bottom - new_height;
	}
	//NOTE:  Should be using virtualscreen_height here but it must be inaccurate
	//      because only wintrack_maxheight lets us keep window from moving way below taskbar!
	if(winbounds->top > (virtualscreen_originy + wintrack_maxheight - 50)) {
		int new_height = winbounds->bottom - winbounds->top;
		winbounds->top = oldbounds->top;
		//keep height as requested
		winbounds->bottom = winbounds->top + new_height;
	}
}

void XWFWchangeGrabbedWindowBounds(XWFWRect *by_amountp, FWHID_HandMotionPtr movinghandp) {
	if(by_amountp != NULL && gGrabbedWindow != NULL) {
		RECT oldbounds, winbounds;//LONG left, top, right, bottom
		//need to deMaximize windows before moving/resizing or they will think they're still maximized!!!
		xwfw_deMaximize(gGrabbedWindow); 

		GetWindowRect(gGrabbedWindow,&winbounds);
		oldbounds = winbounds;
		winbounds.left += (long)floor(by_amountp->origin.x + .5);
		winbounds.top += (long)floor(by_amountp->origin.y + .5);
		winbounds.right += (long)floor(by_amountp->origin.x + by_amountp->size.width + .5);
		winbounds.bottom += (long)floor(by_amountp->origin.y + by_amountp->size.height + .5);
		xwfw_limitWindowBounds(&winbounds, &oldbounds);
		xwfw_preserveCursorRelation(&winbounds, &oldbounds);
		//this may try to restore old size but we already read maximized size so MoveWindow will override
		MoveWindow(gGrabbedWindow, winbounds.left, winbounds.top, 
			winbounds.right - winbounds.left, 
			winbounds.bottom - winbounds.top,true);
	}
}

void XWFWpanWindow(int hscroll) {
	if(hscroll != 0) {
		HWND win2scroll;
		GUITHREADINFO gti;
		gti.cbSize = sizeof(GUITHREADINFO);
		GetGUIThreadInfo(0,&gti);
		win2scroll = gti.hwndFocus;
		if(win2scroll != NULL) {
			while(hscroll < 0) {
				SendMessage(win2scroll, WM_HSCROLL, MAKELONG(SB_LINELEFT, 0), 0L); 
				hscroll++;
			}
			while(hscroll > 0) {
				SendMessage(win2scroll, WM_HSCROLL, MAKELONG(SB_LINERIGHT, 0), 0L); 
				hscroll--;
			}
		}
	}
}



//------ Win32 Messaging-Window/Dialog/StatusMenu creation  Yeah!--------

HMENU gStatusMenu = NULL;
HINSTANCE gHinst = NULL;
NOTIFYICONDATA gNid;

LRESULT CALLBACK xwfw_ShellNotifyProc (HWND hwnd, WPARAM wParam, LPARAM lParam) {    
	POINT current_cursor_pos;
	UINT statusmenu_command = 0;
	GetCursorPos(&current_cursor_pos);
    switch (lParam) {
		case WM_CONTEXTMENU:
		case WM_RBUTTONDOWN:
		case WM_LBUTTONDOWN:
			SetForegroundWindow(hwnd);
			if(gStatusMenu != NULL) {
				TrackPopupMenuEx(GetSubMenu(gStatusMenu,0),
					0,current_cursor_pos.x,current_cursor_pos.y,hwnd,NULL);
			}
		//case WM_CANCEL:
		//	Shell_NotifyIcon(NIM_SETFOCUS,&gNid);
		default:
			return 0;
	}
}

static BOOL xwfw_addNotifyShellIcon(HINSTANCE hInst, HWND hwnd) {
	gNid.cbSize = sizeof(NOTIFYICONDATA); //OK for IE_SHELL versions 5.0 & 6.0
	gNid.hWnd = hwnd;
	gNid.uID = IDI_XWICON16;
	gNid.hIcon = LoadIcon (hInst, MAKEINTRESOURCE(IDI_XWICON16));
	lstrcpy(gNid.szTip,"XWinder");
	gNid.uFlags = NIF_TIP | NIF_ICON | NIF_MESSAGE;
	gNid.uCallbackMessage = WM_XWINDERSHELLICON;
	return Shell_NotifyIcon(NIM_ADD,&gNid);
}

INT_PTR CALLBACK xwfw_AboutDlgProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{ 
    switch (message) { 
		case WM_INITDIALOG:
			return TRUE;
 		case WM_COMMAND:
			if(wParam == IDOK) 
				EndDialog(hwndDlg,0);
        default: 
            return FALSE; 
    } 
}

void xwfw_setOrthoStretchAsDefault(BOOLEAN orthostretch_without5F) {
	MENUITEMINFO mii;
	XWFWsetBooleanPreference("OrthoStretchAsDefault",orthostretch_without5F);
	mii.cbSize = sizeof(MENUITEMINFO);
	mii.fMask = MIIM_STATE;
	if (orthostretch_without5F) {
		mii.fState = MFS_CHECKED;
		SetMenuItemInfo(gStatusMenu,ID_STATUS_ORTHOSTRETCH,false,&mii);
		mii.fState = MFS_UNCHECKED;
		SetMenuItemInfo(gStatusMenu,ID_STATUS_SIZEASPECT,false,&mii);
	} else {
		mii.fState = MFS_UNCHECKED;
		SetMenuItemInfo(gStatusMenu,ID_STATUS_ORTHOSTRETCH,false,&mii);
		mii.fState = MFS_CHECKED;
		SetMenuItemInfo(gStatusMenu,ID_STATUS_SIZEASPECT,false,&mii);
	}
}

void xwfw_setGrabKeyfocusWindow(BOOLEAN use_keyfocus) {
	MENUITEMINFO mii;
	XWFWsetBooleanPreference("UseKeyFocusWindow",use_keyfocus);
	mii.cbSize = sizeof(MENUITEMINFO);
	mii.fMask = MIIM_STATE;
	if (use_keyfocus) {
		mii.fState = MFS_CHECKED;
		SetMenuItemInfo(gStatusMenu,ID_STATUS_GRABKEYFOCUSWINDOW,false,&mii);
		mii.fState = MFS_UNCHECKED;
		SetMenuItemInfo(gStatusMenu,ID_STATUS_GRABWINDOWUNDERMOUSE,false,&mii);
	} else {
		mii.fState = MFS_UNCHECKED;
		SetMenuItemInfo(gStatusMenu,ID_STATUS_GRABKEYFOCUSWINDOW,false,&mii);
		mii.fState = MFS_CHECKED;
		SetMenuItemInfo(gStatusMenu,ID_STATUS_GRABWINDOWUNDERMOUSE,false,&mii);
	}
}

LRESULT CALLBACK xwfw_MsgOnlyWndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lParam) {    
/*    PAINTSTRUCT   ps;             
	HDC hdc;
*/
    switch (nMsg) {
        case WM_DESTROY:
			OutputDebugString (TEXT("WM_DESTROY: stopHandMotionDevices!\n")); 
			FWHID_releaseMultiTouchDevices();
			if(gNid.hWnd != NULL)
				Shell_NotifyIcon(NIM_DELETE,&gNid);
			//if(gStatusMenu != NULL)
				//DestroyMenu(gStatusMenu);
            PostQuitMessage (0);
            return 0;

        /*case WM_PAINT:	    
			hdc = BeginPaint(hwnd, &ps);
			fwhdemo_PaintHIDinfo(hwnd, hdc, &ps);
			EndPaint(hwnd, &ps);
			return 0;*/

		//case WM_ERASEBKGND:
		//	return (LRESULT)1; // Say we handled it.

		case WM_XWINDERSHELLICON:

			//OutputDebugString (TEXT("Got NotifyShellIcon Message!\n")); 
			return xwfw_ShellNotifyProc(hwnd,wParam,lParam);
		case WM_COMMAND:
			if(wParam == ID_XWINDER_EXIT) {
				OutputDebugString (TEXT("Posting WM_CLOSE Message!\n")); 
				PostMessage(hwnd,WM_CLOSE,0,0);
			} else if(wParam == ID_XWINDER_ABOUT) {
				DialogBox(gHinst,MAKEINTRESOURCE(IDD_ABOUTXWINDER_DIALOG),
					hwnd,xwfw_AboutDlgProc);
				OutputDebugString (TEXT("About XWinder!\n")); 
			} else if(wParam == ID_STATUS_GRABKEYFOCUSWINDOW) {
				xwfw_setGrabKeyfocusWindow(true);
			} else if(wParam == ID_STATUS_GRABWINDOWUNDERMOUSE) {
				xwfw_setGrabKeyfocusWindow(false);
			} else if(wParam == ID_STATUS_SIZEASPECT) {
				xwfw_setOrthoStretchAsDefault(false);
			} else if(wParam == ID_STATUS_ORTHOSTRETCH) {
				xwfw_setOrthoStretchAsDefault(true);
			}
	}		

	//IMPORTANT: let Hand Motion library check/process WM_INPUT or WM_DEVICECHANGED messages
	if(FWHID_WndProc (hwnd, nMsg, wParam, lParam))
		return TRUE;

    //if none of above pass to Default Window procedure
	return DefWindowProc (hwnd, nMsg, wParam, lParam);
}

static HMENU xwfw_createStatusMenu(HINSTANCE hInst) {
	HMENU newMenu = LoadMenu(hInst,MAKEINTRESOURCE(IDR_STATUSMENU));
	MENUITEMINFO mii;

	//make sure preferences are all initialized before setting menu state below!
	xwfw_initializePreferences();

	mii.cbSize = sizeof(MENUITEMINFO);
	mii.fMask = MIIM_FTYPE | MIIM_STATE;

	GetMenuItemInfo(newMenu,ID_STATUS_GRABKEYFOCUSWINDOW,false,&mii);
	mii.fType |= MFT_RADIOCHECK;
	if(gUseKeyFocusWindow)
		mii.fState = MFS_CHECKED;
	else mii.fState = MFS_UNCHECKED;
	SetMenuItemInfo(newMenu,ID_STATUS_GRABKEYFOCUSWINDOW,false,&mii);

	GetMenuItemInfo(newMenu,ID_STATUS_GRABWINDOWUNDERMOUSE,false,&mii);
	mii.fType |= MFT_RADIOCHECK;
	if(!gUseKeyFocusWindow)
		mii.fState = MFS_CHECKED;
	else mii.fState = MFS_UNCHECKED;
	SetMenuItemInfo(newMenu,ID_STATUS_GRABWINDOWUNDERMOUSE,false,&mii);

	GetMenuItemInfo(newMenu,ID_STATUS_ORTHOSTRETCH,false,&mii);
	mii.fType |= MFT_RADIOCHECK;
	if(gOrthoStretchAsDefault)
		mii.fState = MFS_CHECKED;
	else mii.fState = MFS_UNCHECKED;
	SetMenuItemInfo(newMenu,ID_STATUS_ORTHOSTRETCH,false,&mii);

	GetMenuItemInfo(newMenu,ID_STATUS_SIZEASPECT,false,&mii);
	mii.fType |= MFT_RADIOCHECK;
	if(!gOrthoStretchAsDefault)
		mii.fState = MFS_CHECKED;
	else mii.fState = MFS_UNCHECKED;
	SetMenuItemInfo(newMenu,ID_STATUS_SIZEASPECT,false,&mii);
			
	return newMenu;
}

static HWND xwfw_createMsgWindow(HINSTANCE hInst, int nShow) {
	HWND         hwndMain;        /* Handle for the main window. */
    WNDCLASSEX   wndclass;        /* A window class structure. */
    char*        fwMainWndClassName = "FWHID Hand Motion Receiver";/* The name of the main window class */	

	gStatusMenu = xwfw_createStatusMenu(hInst);

    memset (&wndclass, 0, sizeof(WNDCLASSEX));
    wndclass.lpszClassName = fwMainWndClassName;
    wndclass.cbSize = sizeof(WNDCLASSEX);
    wndclass.style = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc = xwfw_MsgOnlyWndProc;
    wndclass.hInstance = hInst;
    wndclass.hIcon = LoadIcon (hInst, MAKEINTRESOURCE(IDI_XWICON32));
    wndclass.hIconSm = LoadIcon (hInst, MAKEINTRESOURCE(IDI_XWICON16));
    wndclass.hCursor = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    RegisterClassEx (&wndclass);
    hwndMain = CreateWindow (
            fwMainWndClassName,             /* Class name */
            "Got any Hand Events Lately?",                    /* Caption */
            WS_MINIMIZEBOX | WS_OVERLAPPED | WS_CAPTION |WS_SYSMENU,        /* Style */
            CW_USEDEFAULT,              /* Initial x (use default) */
            CW_USEDEFAULT,              /* Initial y (use default) */
            CW_USEDEFAULT,              /* Initial width (use default) */
            CW_USEDEFAULT,              /* Initial height (use default) */
            HWND_MESSAGE,                       /* NULL for No parent window or HWND_MESSAGE for message-only */
            gStatusMenu,                       /* No menu */
            hInst,                      /* This program instance */
            NULL                        /* Creation parameters */
            );
	if(!hwndMain)
		return NULL;

	//XWinder only has a status icon for now.
	//Maybe someday we'll put some cool animation in a little application window!
	//Anyway, don't show!
	ShowWindow (hwndMain, nShow);
    UpdateWindow (hwndMain);
	return hwndMain;
}


int APIENTRY 
WinMain (HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmd, int nShow) {
		 
    MSG          msg;             /* A Win32 message structure. */
	BOOL bRet;
	
    if (g_SingleInstanceObj.IsAnotherInstanceRunning())
       return FALSE; 

	gHinst = hInst;
	gHwndMain = xwfw_createMsgWindow(hInst, nShow);	
	xwfw_addNotifyShellIcon(hInst,gHwndMain);

	FWHID_registerMultiTouchDevices(gHwndMain);
    FWHID_setHandEventCallback(XWFWhandleHandMotion);

	while( (bRet = GetMessage( &msg, NULL, 0, 0 )) != 0) { 
		if (bRet == -1) {
			// handle the error and possibly exit
		} else {
			TranslateMessage(&msg); 
			DispatchMessage(&msg); 
		}
	}

    return (int)msg.wParam;
}

#ifdef __cplusplus
}
#endif
